"""Reload the OS on a virtual server."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--postinstall', '-i', help="Post-install script to download")
@click.option(
    '--image',
    help="""Image ID. The default is to use the current operating system.
See: 'slcli image list' for reference""")
@helpers.multi_option('--key', '-k',
                      help="SSH keys to add to the root user")
@environment.pass_env
def cli(env, identifier, postinstall, key, image):
    """Reload operating system on a virtual server."""

    vsi = SoftLayer.VSManager(env.client)
    vs_id = helpers.resolve_id(vsi.resolve_ids, identifier, 'VS')
    keys = []
    if key:
        for single_key in key:
            resolver = SoftLayer.SshKeyManager(env.client).resolve_ids
            key_id = helpers.resolve_id(resolver, single_key, 'SshKey')
            keys.append(key_id)
    if not (env.skip_confirmations or formatting.no_going_back(vs_id)):
        raise exceptions.CLIAbort('Aborted')

    vsi.reload_instance(vs_id,
                        post_uri=postinstall,
                        ssh_keys=keys,
                        image_id=image)



"""Authorize File, Block and Portable Storage to a Virtual Server"""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--username-storage', '-u', type=click.STRING,
              help="The storage username to be added to the virtual server")
@click.option('--portable-id', type=click.INT,
              help="The portable storage id to be added to the virtual server")
@environment.pass_env
def cli(env, identifier, username_storage, portable_id):
    """Authorize File, Block and Portable Storage to a Virtual Server."""
    virtual = SoftLayer.VSManager(env.client)
    virtual_id = helpers.resolve_id(virtual.resolve_ids, identifier, 'virtual')
    table = formatting.KeyValueTable(['name', 'value'], title="Portable Storage Detail")
    table.align['name'] = 'r'
    table.align['value'] = 'l'

    if username_storage:
        if not virtual.authorize_storage(virtual_id, username_storage):
            raise exceptions.CLIAbort('Authorize Volume Failed')
        env.fout(f'Successfully Volume: {username_storage} was Added.')
    if portable_id:
        portable_id = helpers.resolve_id(virtual.resolve_ids, portable_id, 'storage')
        portable_result = virtual.attach_portable_storage(virtual_id, portable_id)

        env.fout(f'Successfully Portable Storage: {portable_id} was Added.')

        table.add_row(['Id', portable_result['id']])
        table.add_row(['createDate', portable_result['createDate']])
        env.fout(table)



"""Failover to a replicant volume."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume-id')
@click.option('--replicant-id', help="ID of the replicant volume")
@environment.pass_env
def cli(env, volume_id, replicant_id):
    """Failover a file volume to the given replicant volume.

    Example::
        slcli file replica-failover 12345678 87654321
        This command performs failover operation for volume with ID 12345678 to replica volume with ID 87654321.
"""
    file_storage_manager = SoftLayer.FileStorageManager(env.client)

    success = file_storage_manager.failover_to_replicant(
        volume_id,
        replicant_id
    )

    if success:
        click.echo("Failover to replicant is now in progress.")
    else:
        click.echo("Failover operation could not be initiated.")



"""List Object Storage accounts."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--limit',
              type=int,
              default=10,
              help="Result limit")
@environment.pass_env
def cli(env, limit):
    """List object storage accounts."""

    mgr = SoftLayer.ObjectStorageManager(env.client)
    accounts = mgr.list_accounts(limit=limit)
    table = formatting.Table(['id', 'name', 'apiType'])
    table.sortby = 'id'
    api_type = None
    for account in accounts:
        if 'vendorName' in account and account['vendorName'] == 'Swift':
            api_type = 'Swift'
        elif 'Cleversafe' in account['serviceResource']['name']:
            api_type = 'S3'

        table.add_row([
            account['id'],
            account['username'],
            api_type,
        ])

    env.fout(table)



"""Utility functions for use with File and Block commands."""
# :license: MIT, see LICENSE for more details.

from SoftLayer.CLI import columns as column_helper
from SoftLayer.CLI import formatting

DEFAULT_NOTES_SIZE = 20


def reduce_notes(volumes, env):
    """Reduces all long notes found in the volumes list just if the format output is different from a JSON format.

    :param list volumes: An list of storage volumes
    :param env :A environment console.
    """
    if env.format_output_is_json():
        return

    for volume in volumes:
        if len(volume.get('notes', '')) > DEFAULT_NOTES_SIZE:
            shortened_notes = volume['notes'][:DEFAULT_NOTES_SIZE]
            volume['notes'] = shortened_notes


def build_output_table(env, volumes, columns, sortby):
    """Builds a formatting table for a list of volumes.

        :param env :A Environment console.
        :param list volumes: An list of storage volumes
        :param columns :A ColumnFormatter for column names
        :param str sortby :A string to sort by.
        """
    table = formatting.Table(columns.columns)
    if sortby in table.columns:
        table.sortby = sortby

    reduce_notes(volumes, env)
    for volume in volumes:
        table.add_row([value or formatting.blank()
                       for value in columns.row(volume)])
    return table


def _format_name(obj):
    if obj['type'] == 'VIRTUAL':
        return f"{obj['hostname']}.{obj['domain']}"

    elif obj['type'] == 'HARDWARE':
        return f"{obj['hostname']}.{obj['domain']}"

    elif obj['type'] == 'SUBNET':
        name = f"{obj['networkIdentifier']}/{obj['cidr']}"
        if 'note' in obj.keys():
            name = f"{name} ({obj['note']})"

        return name

    elif obj['type'] == 'IP':
        name = obj['ipAddress']
        if 'note' in obj.keys():
            name = f"{name} ({obj['note']})"

        return name
    else:
        raise ValueError('Unknown type %s' % obj['type'])


COLUMNS = [
    column_helper.Column('id', ('id',)),
    column_helper.Column('name', _format_name, """
allowedVirtualGuests[hostname,domain],
allowedHardware[hostname,domain],
allowedSubnets[networkIdentifier,cidr,note],
allowedIpAddresses[ipAddress,note],
"""),
    column_helper.Column('type', ('type',)),
    column_helper.Column(
        'private_ip_address',
        ('primaryBackendIpAddress',),
        """
allowedVirtualGuests.primaryBackendIpAddress
allowedHardware.primaryBackendIpAddress
allowedSubnets.primaryBackendIpAddress
allowedIpAddresses.primaryBackendIpAddress
"""),
    column_helper.Column(
        'source_subnet',
        ('allowedHost', 'sourceSubnet',),
        """
allowedVirtualGuests.allowedHost.sourceSubnet
allowedHardware.allowedHost.sourceSubnet
allowedSubnets.allowedHost.sourceSubnet
allowedIpAddresses.allowedHost.sourceSubnet
"""),
    column_helper.Column(
        'host_iqn',
        ('allowedHost', 'name',),
        """
allowedVirtualGuests.allowedHost.name
allowedHardware.allowedHost.name
allowedSubnets.allowedHost.name
allowedIpAddresses.allowedHost.name
"""),
    column_helper.Column(
        'username',
        ('allowedHost', 'credential', 'username',),
        """
allowedVirtualGuests.allowedHost.credential.username
allowedHardware.allowedHost.credential.username
allowedSubnets.allowedHost.credential.username
allowedIpAddresses.allowedHost.credential.username
"""),
    column_helper.Column(
        'password',
        ('allowedHost', 'credential', 'password',),
        """
allowedVirtualGuests.allowedHost.credential.password
allowedHardware.allowedHost.credential.password
allowedSubnets.allowedHost.credential.password
allowedIpAddresses.allowedHost.credential.password
"""),
    column_helper.Column(
        'allowed_host_id',
        ('allowedHost', 'id',),
        """
allowedVirtualGuests.allowedHost.id
allowedHardware.allowedHost.id
allowedSubnets.allowedHost.id
allowedIpAddresses.allowedHost.id
"""),
]

DEFAULT_COLUMNS = [
    'id',
    'name',
    'type',
    'private_ip_address',
    'source_subnet',
    'host_iqn',
    'username',
    'password',
    'allowed_host_id',
]


REPLICATION_PARTNER_COLUMNS = [
    column_helper.Column('ID', ('id',)),
    column_helper.Column('Username', ('username',), mask="username"),
    column_helper.Column('Account ID', ('accountId',), mask="accountId"),
    column_helper.Column('Capacity (GB)', ('capacityGb',), mask="capacityGb"),
    column_helper.Column('Hardware ID', ('hardwareId',), mask="hardwareId"),
    column_helper.Column('Guest ID', ('guestId',), mask="guestId"),
    column_helper.Column('Host ID', ('hostId',), mask="hostId"),
]

REPLICATION_PARTNER_DEFAULT = [
    'ID',
    'Username',
    'Account ID',
    'Capacity (GB)',
    'Hardware ID',
    'Guest ID',
    'Host ID'
]



getByIpAddress = {
    'id': 12345,
    'ipAddress': '10.0.1.37',
    'isBroadcast': False,
    'isGateway': False,
    'isNetwork': False,
    'isReserved': False,
    'subnetId': 5678,
    "hardware": {
        "id": 12856,
        "fullyQualifiedDomainName": "unit.test.com"
    },
    "subnet": {
        "broadcastAddress": "10.0.1.91",
        "cidr": 26,
        "gateway": "10.47.16.129",
        "id": 258369,
        "isCustomerOwned": False,
        "isCustomerRoutable": False,
        "modifyDate": "2019-04-02T13:45:52-06:00",
        "netmask": "255.255.255.192",
        "networkIdentifier": "10.0.1.38",
        "networkVlanId": 1236987,
        "sortOrder": "0",
        "subnetType": "PRIMARY",
        "totalIpAddresses": "64",
        "usableIpAddressCount": "61",
        "version": 4
    }
}

editObject = True



"""Update DNS record."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import helpers
# pylint: disable=redefined-builtin


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('zone-id')
@click.option('--by-record', help='Edit by host record, such as www')
@click.option('--by-id', help='Edit a single record by its ID')
@click.option('--data', help='Record data, such as an IP address')
@click.option('--ttl',
              type=click.INT,
              help='TTL value in seconds, such as 86400')
@environment.pass_env
def cli(env, zone_id, by_record, by_id, data, ttl):
    """Update DNS record.

    Example::
        slcli dns record-edit ibm.com --by-id 12345678 --data 127.0.0.2 --ttl 3600
        This command edits records under the zone: ibm.com, whose ID is 12345678, \
and sets its data to "127.0.0.2" and ttl to 3600.

        slcli dns record-edit ibm.com --by-record kibana --ttl 3600
        This command edits records under the zone: ibm.com, whose host is "kibana", and sets their ttl all to 3600.
"""
    manager = SoftLayer.DNSManager(env.client)
    zone_id = helpers.resolve_id(manager.resolve_ids, zone_id, name='zone')

    results = manager.get_records(zone_id, host=by_record)

    for result in results:
        if by_id and str(result['id']) != by_id:
            continue
        result['data'] = data or result['data']
        result['ttl'] = ttl or result['ttl']
        manager.edit_record(result)



"""Creates a user """
# :license: MIT, see LICENSE for more details.

import json
import string
import sys

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('username')
@click.option('--email', '-e', required=True,
              help="Email address for this user. Required for creation.")
@click.option('--password', '-p', default=None, show_default=True,
              help="Password to set for this user. If no password is provided, user will be sent an email "
                   "to generate one, which expires in 24 hours.  '-p generate' will create a password for you "
                   "(Requires Python 3.6+). Passwords require 8+ characters, upper and lowercase, a number "
                   "and a symbol.")
@click.option('--from-user', '-u', default=None,
              help="Base user to use as a template for creating this user. "
                   "Will default to the user running this command. Information provided in --template "
                   "supersedes this template.")
@click.option('--template', '-t', default=None,
              help="A json string describing https://softlayer.github.io/reference/datatypes/SoftLayer_User_Customer/")
@environment.pass_env
def cli(env, username, email, password, from_user, template):
    """Creates a user Users.

    Remember to set the permissions and access for this new user.

    Example::

        slcli user create my@email.com -e my@email.com -p generate -a
        -t '{"firstName": "Test", "lastName": "Testerson"}'

    """

    mgr = SoftLayer.UserManager(env.client)
    user_mask = ("mask[id, firstName, lastName, email, companyName, address1, city, country, postalCode, "
                 "state, userStatusId, timezoneId]")
    from_user_id = None
    if from_user is None:
        user_template = mgr.get_current_user(objectmask=user_mask)
        from_user_id = user_template['id']
    else:
        from_user_id = helpers.resolve_id(mgr.resolve_ids, from_user, 'username')
        user_template = mgr.get_user(from_user_id, objectmask=user_mask)
    # If we send the ID back to the API, an exception will be thrown
    del user_template['id']

    if template is not None:
        try:
            template_object = json.loads(template)
            for key in template_object:
                user_template[key] = template_object[key]
        except ValueError as ex:
            raise exceptions.ArgumentError(f"Unable to parse --template. {ex}")

    user_template['username'] = username
    if password == 'generate':
        password = generate_password()

    user_template['email'] = email

    if not env.skip_confirmations:
        table = formatting.KeyValueTable(['name', 'value'])
        for key in user_template:
            table.add_row([key, user_template[key]])
        table.add_row(['password', password])
        click.secho("You are about to create the following user...", fg='green')
        env.fout(table)
        if not formatting.confirm("Do you wish to continue?"):
            raise exceptions.CLIAbort("Canceling creation!")

    result = mgr.create_user(user_template, password)

    table = formatting.Table(['Username', 'Email', 'Password'])
    table.add_row([result['username'], result['email'], password])
    env.fout(table)


def generate_password():
    """Returns a 23 character random string, with 3 special characters at the end"""
    if sys.version_info > (3, 6):
        import secrets  # pylint: disable=import-error,import-outside-toplevel
        alphabet = string.ascii_letters + string.digits
        password = ''.join(secrets.choice(alphabet) for i in range(20))
        special = ''.join(secrets.choice(string.punctuation) for i in range(3))
        return password + special
    else:
        raise ImportError("Generating passwords require python 3.6 or higher")



"""
    SoftLayer.transports.fixture
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
    Fixture transport, used for unit tests

    :license: MIT, see LICENSE for more details.
"""

import importlib

from .transport import SoftLayerListResult


class FixtureTransport(object):
    """Implements a transport which returns fixtures."""

    def __call__(self, call):
        """Load fixture from the default fixture path."""
        try:
            module_path = 'SoftLayer.fixtures.%s' % call.service
            module = importlib.import_module(module_path)
        except ImportError as ex:
            message = f'{call.service} fixture is not implemented'
            raise NotImplementedError(message) from ex
        try:
            result = getattr(module, call.method)
            if isinstance(result, list):
                return SoftLayerListResult(result, len(result))
            return result
        except AttributeError as ex:
            message = f'{call.service}::{call.method} fixture is not implemented'
            raise NotImplementedError(message) from ex

    @staticmethod
    def print_reproduceable(call):
        """Not Implemented"""
        return call.service



"""
    SoftLayer.vs_placement
    ~~~~~~~~~~~~~~~~~~~~~~~
    Placement Group Manager

    :license: MIT, see License for more details.
"""

import logging

from SoftLayer import utils

# Invalid names are ignored due to long method names and short argument names
# pylint: disable=invalid-name

LOGGER = logging.getLogger(__name__)


class PlacementManager(utils.IdentifierMixin, object):
    """Manages SoftLayer Reserved Capacity Groups.

        Product Information

        - https://console.test.cloud.ibm.com/docs/vsi/vsi_placegroup.html#placement-groups
        - https://softlayer.github.io/reference/services/SoftLayer_Account/getPlacementGroups/
        - https://softlayer.github.io/reference/services/SoftLayer_Virtual_PlacementGroup_Rule/

        Existing instances cannot be added to a placement group.
        You can only add a virtual server instance to a placement group at provisioning.
        To remove an instance from a placement group, you must delete or reclaim the instance.

    :param SoftLayer.API.BaseClient client: the client instance
    """

    def __init__(self, client):
        self.client = client
        self.account = client['Account']
        self.resolvers = [self._get_id_from_name]

    def list(self, mask=None):
        """List existing placement groups

        Calls SoftLayer_Account::getPlacementGroups
         """
        if mask is None:
            mask = "mask[id, name, createDate, rule, guestCount, backendRouter[id, hostname]]"
        groups = self.client.call('Account', 'getPlacementGroups', mask=mask, iter=True)
        return groups

    def create(self, placement_object):
        """Creates a placement group

        A placement_object is defined as::

            placement_object = {
                'backendRouterId': 12345,
                'name': 'Test Name',
                'ruleId': 12345
            }

        -  https://softlayer.github.io/reference/datatypes/SoftLayer_Virtual_PlacementGroup/

        :param dictionary placement_object:

        """
        return self.client.call('SoftLayer_Virtual_PlacementGroup', 'createObject', placement_object)

    def get_routers(self):
        """Calls SoftLayer_Virtual_PlacementGroup::getAvailableRouters()"""
        return self.client.call('SoftLayer_Virtual_PlacementGroup', 'getAvailableRouters')

    def get_object(self, group_id, mask=None):
        """Returns a PlacementGroup Object

        https://softlayer.github.io/reference/services/SoftLayer_Virtual_PlacementGroup/getObject
        """
        if mask is None:
            mask = "mask[id, name, createDate, rule, backendRouter[id, hostname]," \
                   "guests[activeTransaction[id,transactionStatus[name,friendlyName]]]]"
        return self.client.call('SoftLayer_Virtual_PlacementGroup', 'getObject', id=group_id, mask=mask)

    def delete(self, group_id):
        """Deletes a PlacementGroup

        Placement group must be empty to be deleted.
        https://softlayer.github.io/reference/services/SoftLayer_Virtual_PlacementGroup/deleteObject
        """
        return self.client.call('SoftLayer_Virtual_PlacementGroup', 'deleteObject', id=group_id)

    def get_all_rules(self):
        """Returns all available rules for creating a placement group"""
        return self.client.call('SoftLayer_Virtual_PlacementGroup_Rule', 'getAllObjects')

    def get_rule_id_from_name(self, name):
        """Finds the rule that matches name.

        SoftLayer_Virtual_PlacementGroup_Rule.getAllObjects doesn't support objectFilters.
        """
        results = self.client.call('SoftLayer_Virtual_PlacementGroup_Rule', 'getAllObjects')
        return [result['id'] for result in results if result['keyName'] == name.upper()]

    def get_backend_router_id_from_hostname(self, hostname):
        """Finds the backend router Id that matches the hostname given

        No way to use an objectFilter to find a backendRouter, so we have to search the hard way.
        """
        results = self.client.call('SoftLayer_Network_Pod', 'getAllObjects')
        return [result['backendRouterId'] for result in results if result['backendRouterName'] == hostname.lower()]

    def _get_id_from_name(self, name):
        """List placement group ids which match the given name."""
        _filter = {
            'placementGroups': {
                'name': {'operation': name}
            }
        }
        mask = "mask[id, name]"
        results = self.client.call('Account', 'getPlacementGroups', filter=_filter, mask=mask)
        return [result['id'] for result in results]



"""Edit a vlan's details."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--name', '-n',
              help="The optional name for this VLAN")
@click.option('--note', '-e',
              help="The note for this vlan.")
@click.option('--tags', '-g',
              multiple=True,
              help='Tags to set e.g. "tag1,tag2", or empty string to remove all'
              )
@environment.pass_env
def cli(env, identifier, name, note, tags):
    """Edit a vlan's details."""

    if not any([name, note, tags]):
        raise exceptions.CLIAbort("At least one option is required")

    new_tags = None

    if tags:
        new_tags = ','.join(tags)

    mgr = SoftLayer.NetworkManager(env.client)
    vlan_id = helpers.resolve_id(mgr.resolve_vlan_ids, identifier, 'VLAN')
    vlan = mgr.edit(vlan_id, name=name, note=note, tags=new_tags)

    if vlan:
        click.secho("Vlan edited successfully", fg='green')
    else:
        click.secho("Failed to edit the vlan", fg='red')



getBillingItem = {'id': 21370814}
getObject = {
    'accountId': 307608,
    'connectionLimit': 500,
    'connectionLimitUnits': "connections/second",
    'dedicatedFlag': False,
    'highAvailabilityFlag': False,
    'id': 22348,
    'ipAddressId': 7303278,
    'managedResourceFlag': False,
    'sslActiveFlag': False,
    'sslEnabledFlag': True,
    'virtualServers': [
        {
            'allocation': 10,
            'id': 50718,
            'port': 80,
            "serviceGroups": [
                {
                    'id': 51758,
                    'routingMethodId': 10,
                    'routingTypeId': 3,
                    'services': [
                        {
                            'enabled': 1,
                            'id': 1234,
                            'healthChecks': [
                                {
                                    'id': 112112
                                }
                            ],
                            'groupReferences': [
                                {
                                    'serviceGroupId': 51758,
                                    'serviceId': 84986,
                                    'weight': 2
                                }
                            ],
                            'ipAddressId': 14288108,
                            'port': 8080,
                            'status': "DOWN"
                        }
                    ]
                }
            ],
            "virtualIpAddress": {
                'accountId': 307608,
                'connectionLimit': 500,
                'connectionLimitUnits': "connections/second",
                'id': 22348,
                'ipAddressId': 7303278,
            },
            'virtualIpAddressId': 22348
        }]}
getVirtualServers = [
    {
        "allocation": 10,
        "id": 50718,
        "port": 80,
        "serviceGroups": [
            {
                "id": 51758,
                "routingMethodId": 10,
                "routingTypeId": 3,
                "services": [
                    {
                        "enabled": 1,
                        "id": 1234,
                        "healthChecks": [
                            {
                                "id": 112112
                            }
                        ],
                        "groupReferences": [
                            {
                                "serviceGroupId": 51758,
                                "serviceId": 84986,
                                "weight": 2
                            }
                        ],
                        "ipAddressId": 14288108,
                        "port": 8080,
                        "status": "DOWN"
                    }
                ]
            }
        ],
        "virtualIpAddress": {
            "accountId": 307608,
            "connectionLimit": 500,
            "connectionLimitUnits": "connections/second",
            "id": 22348,
            "ipAddressId": 7303278,
        },
        "virtualIpAddressId": 22348
    }
]
editObject = True



"""List number of block storage volumes limit per datacenter."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting

DEFAULT_COLUMNS = [
    'Datacenter',
    'MaximumAvailableCount',
    'ProvisionedCount'
]


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--sortby', help='Column to sort by', default='Datacenter')
@click.option('--datacenter', '-d', help='Filter by datacenter')
@environment.pass_env
def cli(env, sortby, datacenter):
    """List number of block storage volumes limit per datacenter.

    Example::
        slcli block volume-limits
        This command lists the storage limits per datacenter for this account.
    """

    block_manager = SoftLayer.BlockStorageManager(env.client)
    block_volumes = block_manager.list_block_volume_limit()

    table = formatting.KeyValueTable(DEFAULT_COLUMNS)
    table.sortby = sortby

    for volumen in block_volumes:
        if datacenter:
            if volumen.get('datacenterName') != '':
                if volumen.get('datacenterName') == datacenter:
                    table.add_row([volumen.get('datacenterName'),
                                   volumen.get('maximumAvailableCount'),
                                   volumen.get('provisionedCount')])
                    break
        else:
            if volumen.get('datacenterName') != '':
                table.add_row([volumen.get('datacenterName'), volumen.get('maximumAvailableCount'),
                               volumen.get('provisionedCount')])
            else:
                table.add_row([' - ',
                               volumen.get('maximumAvailableCount'),
                               volumen.get('provisionedCount')])
    env.fout(table)



"""Enable or Disable vpn subnets manual config for a user."""
# :license: MIT, see LICENSE for more details.


import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('user')
@click.option('--enable/--disable', default=True,
              help="Enable or disable vpn subnets manual config.")
@environment.pass_env
def cli(env, user, enable):
    """Enable or disable user vpn subnets manual config"""
    mgr = SoftLayer.UserManager(env.client)
    user_id = helpers.resolve_id(mgr.resolve_ids, user, 'username')

    result = mgr.vpn_manual(user_id, enable)
    message = f"{user} vpn manual config {'enable' if enable else 'disable'}"

    if result:
        click.secho(message, fg='green')
    else:
        click.secho(f"Failed to update {user}", fg='red')



"""List file storage snapshots."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import columns as column_helper
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers

COLUMNS = [
    column_helper.Column('id', ('id',), mask='id'),
    column_helper.Column('name', ('notes',), mask='notes'),
    column_helper.Column('created', ('snapshotCreationTimestamp',),
                         mask='snapshotCreationTimestamp'),
    column_helper.Column('size_bytes', ('snapshotSizeBytes',),
                         mask='snapshotSizeBytes'),
]

DEFAULT_COLUMNS = [
    'id',
    'name',
    'created',
    'size_bytes'
]


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume_id')
@click.option('--sortby', help='Column to sort by',
              default='created')
@click.option('--columns',
              callback=column_helper.get_formatter(COLUMNS),
              help=f"Columns to display. Options: {', '.join(column.name for column in COLUMNS)}",
              default=','.join(DEFAULT_COLUMNS))
@environment.pass_env
def cli(env, volume_id, sortby, columns):
    """List file storage snapshots.

    Example::
    slcli file snapshot-list 12345678 --sortby id
    This command lists all snapshots of volume with ID 12345678 and sorts them by ID.
"""
    file_manager = SoftLayer.FileStorageManager(env.client)
    resolved_id = helpers.resolve_id(file_manager.resolve_ids, volume_id, 'Volume Id')
    snapshots = file_manager.get_file_volume_snapshot_list(
        resolved_id,
        mask=columns.mask()
    )

    table = formatting.Table(columns.columns)
    table.sortby = sortby

    for snapshot in snapshots:
        table.add_row([value or formatting.blank()
                       for value in columns.row(snapshot)])

    env.fout(table)



"""Set Tags."""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.managers.tags import TagManager


@click.command(cls=SLCommand)
@click.option('--tags', '-t', type=click.STRING, required=True,
              help='Comma seperated list of tags, enclosed in quotes. "tag1, tag2"')
@click.option('--key-name', '-k', type=click.STRING, required=True,
              help="Key name of a tag type e.g. GUEST, HARDWARE. See slcli tags taggable output.")
@click.option('--resource-id', '-r', type=click.INT, required=True, help="ID of the object being tagged")
@environment.pass_env
def cli(env, tags, key_name, resource_id):
    """Set Tags."""

    tag_manager = TagManager(env.client)
    tags = tag_manager.set_tags(tags, key_name, resource_id)

    if tags:
        click.secho("Set tags successfully", fg='green')
    else:
        click.secho("Failed to set tags", fg='red')



getAllObjects = [
    {
        'accountId': 100,
        'eventCreateDate': '2017-10-23T14:22:36.221541-05:00',
        'eventName': 'Disable Port',
        'ipAddress': '192.168.0.1',
        'label': 'test.softlayer.com',
        'metaData': '',
        'objectId': 300,
        'objectName': 'CCI',
        'traceId': '100',
        'userId': '',
        'userType': 'SYSTEM'
    },
    {
        'accountId': 100,
        'eventCreateDate': '2017-10-18T09:40:41.830338-05:00',
        'eventName': 'Security Group Rule Added',
        'ipAddress': '192.168.0.1',
        'label': 'test.softlayer.com',
        'metaData': '{"securityGroupId":"200",'
                    '"securityGroupName":"test_SG",'
                    '"networkComponentId":"100",'
                    '"networkInterfaceType":"public",'
                    '"requestId":"53d0b91d392864e062f4958",'
                    '"rules":[{"ruleId":"100",'
                    '"remoteIp":null,"remoteGroupId":null,"direction":"ingress",'
                    '"ethertype":"IPv4",'
                    '"portRangeMin":2000,"portRangeMax":2001,"protocol":"tcp"}]}',
        'objectId': 300,
        'objectName': 'CCI',
        'traceId': '59e767e9c2184',
        'userId': 400,
        'userType': 'CUSTOMER',
        'username': 'user'
    },
    {
        'accountId': 100,
        'eventCreateDate': '2017-10-18T09:40:32.238869-05:00',
        'eventName': 'Security Group Added',
        'ipAddress': '192.168.0.1',
        'label': 'test.softlayer.com',
        'metaData': '{"securityGroupId":"200",'
                    '"securityGroupName":"test_SG",'
                    '"networkComponentId":"100",'
                    '"networkInterfaceType":"public",'
                    '"requestId":"96c9b47b9e102d2e1d81fba"}',
        'objectId': 300,
        'objectName': 'CCI',
        'traceId': '59e767e03a57e',
        'userId': 400,
        'userType': 'CUSTOMER',
        'username': 'user'
    },
    {
        'accountId': 100,
        'eventCreateDate': '2017-10-18T10:42:13.089536-05:00',
        'eventName': 'Security Group Rule(s) Removed',
        'ipAddress': '192.168.0.1',
        'label': 'test_SG',
        'metaData': '{"requestId":"2abda7ca97e5a1444cae0b9",'
                    '"rules":[{"ruleId":"800",'
                    '"remoteIp":null,"remoteGroupId":null,"direction":"ingress",'
                    '"ethertype":"IPv4",'
                    '"portRangeMin":2000,"portRangeMax":2001,"protocol":"tcp"}]}',
        'objectId': 700,
        'objectName': 'Security Group',
        'traceId': '59e7765515e28',
        'userId': 400,
        'userType': 'CUSTOMER',
        'username': 'user'
    },
    {
        'accountId': 100,
        'eventCreateDate': '2017-10-18T10:42:11.679736-05:00',
        'eventName': 'Network Component Removed from Security Group',
        'ipAddress': '192.168.0.1',
        'label': 'test_SG',
        'metaData': '{"requestId":"6b9a87a9ab8ac9a22e87a00",'
                    '"fullyQualifiedDomainName":"test.softlayer.com",'
                    '"networkComponentId":"100",'
                    '"networkInterfaceType":"public"}',
        'objectId': 700,
        'objectName': 'Security Group',
        'traceId': '59e77653a1e5f',
        'userId': 400,
        'userType': 'CUSTOMER',
        'username': 'user'
    },
    {
        'accountId': 100,
        'eventCreateDate': '2017-10-18T10:41:49.802498-05:00',
        'eventName': 'Security Group Rule(s) Added',
        'ipAddress': '192.168.0.1',
        'label': 'test_SG',
        'metaData': '{"requestId":"0a293c1c3e59e4471da6495",'
                    '"rules":[{"ruleId":"800",'
                    '"remoteIp":null,"remoteGroupId":null,"direction":"ingress",'
                    '"ethertype":"IPv4",'
                    '"portRangeMin":2000,"portRangeMax":2001,"protocol":"tcp"}]}',
        'objectId': 700,
        'objectName': 'Security Group',
        'traceId': '59e7763dc3f1c',
        'userId': 400,
        'userType': 'CUSTOMER',
        'username': 'user'
    },
    {
        'accountId': 100,
        'eventCreateDate': '2017-10-18T10:41:42.176328-05:00',
        'eventName': 'Network Component Added to Security Group',
        'ipAddress': '192.168.0.1',
        'label': 'test_SG',
        'metaData': '{"requestId":"4709e02ad42c83f80345904",'
                    '"fullyQualifiedDomainName":"test.softlayer.com",'
                    '"networkComponentId":"100",'
                    '"networkInterfaceType":"public"}',
        'objectId': 700,
        'objectName': 'Security Group',
        'traceId': '59e77636261e7',
        'userId': 400,
        'userType': 'CUSTOMER',
        'username': 'user'
    }
]

getAllEventObjectNames = [
    {
        'value': "Account"
    },
    {
        'value': "CDN"
    },
    {
        'value': "User"
    },
    {
        'value': "Bare Metal Instance"
    },
    {
        'value': "API Authentication"
    },
    {
        'value': "Server"
    },
    {
        'value': "CCI"
    },
    {
        'value': "Image"
    },
    {
        'value': "Bluemix LB"
    },
    {
        'value': "Facility"
    },
    {
        'value': "Cloud Object Storage"
    },
    {
        'value': "Security Group"
    }
]



"""Shows the details of a reserved capacity group"""

import click

from SoftLayer.CLI import columns as column_helper
from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers.vs_capacity import CapacityManager as CapacityManager

COLUMNS = [
    column_helper.Column('Id', ('id',)),
    column_helper.Column('hostname', ('hostname',)),
    column_helper.Column('domain', ('domain',)),
    column_helper.Column('primary_ip', ('primaryIpAddress',)),
    column_helper.Column('backend_ip', ('primaryBackendIpAddress',)),
]

DEFAULT_COLUMNS = [
    'id',
    'hostname',
    'domain',
    'primary_ip',
    'backend_ip'
]


@click.command(cls=SLCommand, epilog="Once provisioned, virtual guests can be managed with the slcli vs commands")
@click.argument('identifier')
@click.option('--columns',
              callback=column_helper.get_formatter(COLUMNS),
              help=f"Columns to display. [options: {', '.join(column.name for column in COLUMNS)}]",
              default=','.join(DEFAULT_COLUMNS),
              show_default=True)
@environment.pass_env
def cli(env, identifier, columns):
    """Reserved Capacity Group details. Will show which guests are assigned to a reservation."""

    manager = CapacityManager(env.client)
    mask = """mask[instances[id,createDate,guestId,billingItem[id, description, recurringFee, category[name]],
              guest[modifyDate,id, primaryBackendIpAddress, primaryIpAddress,domain, hostname]]]"""
    result = manager.get_object(identifier, mask)

    try:
        flavor = result['instances'][0]['billingItem']['description']
    except KeyError:
        flavor = "Pending Approval..."

    table = formatting.Table(columns.columns, title=f"{result.get('name')} - {flavor}")
    # RCI = Reserved Capacity Instance
    for rci in result['instances']:
        guest = rci.get('guest', None)
        if guest is not None:
            table.add_row([value or formatting.blank() for value in columns.row(guest)])
        else:
            table.add_row(['-' for value in columns.columns])
    env.fout(table)



"""Reflash firmware."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """Reflash server firmware."""

    mgr = SoftLayer.HardwareManager(env.client)
    hw_id = helpers.resolve_id(mgr.resolve_ids, identifier, 'hardware')
    if not (env.skip_confirmations or
            formatting.confirm('This will power off the server with id %s and '
                               'reflash device firmware. Continue?' % hw_id)):
        raise exceptions.CLIAbort('Aborted.')

    if mgr.reflash_firmware(hw_id):
        click.echo('Successfully device firmware reflashed')



"""Login with your employee username, password, 2fa token"""
# :license: MIT, see LICENSE for more details.
import os

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer import config


def censor_password(value):
    """Replaces a password with *s"""
    if value:
        value = '*' * len(value)
    return value


@click.command(cls=SLCommand)
@click.option('--session-token',
              default=None,
              help='An existing employee session token (hash). Click the "Copy Session Token" in the internal portal '
                   'to get this value.'
                   'Can also be set via the SLCLI_SESSION_TOKEN environment variable.',
              envvar='SLCLI_SESSION_TOKEN')
@click.option('--user-id',
              default=None,
              type=int,
              help='Employee IMS ID. The number in the url when you click your username in the internal portal, '
                   'under "user information". Can also be set via the SLCLI_USER_ID environment variable. '
                   'Or read from the configuration file.',
              envvar='SLCLI_USER_ID')
@click.option('--legacy',
              default=False,
              type=bool,
              is_flag=True,
              help='Login with username, password, yubi key combination. Only valid if ISV is not required. '
                   'If using ISV, use your session token.')
@environment.pass_env
def cli(env, session_token, user_id, legacy):
    """Logs you into the internal SoftLayer Network.

    username: Set this in either the softlayer config, or SL_USER ENV variable
    password: Set this in SL_PASSWORD env variable. You will be prompted for them otherwise.

    To log in with an existing session token instead of username/password/2FA:

        slcli login --session-token <token> --user-id <id>

    Or via environment variables:

        SLCLI_SESSION_TOKEN=<token> SLCLI_USER_ID=<id> slcli login
    """
    config_settings = config.get_config(config_file=env.config_file)
    settings = config_settings['softlayer']

    if not user_id:
        user_id = int(settings.get('userid', 0)) or int(os.environ.get('SLCLI_USER_ID', 0))
    # --session-token supplied on the CLI (or via SLCLI_SESSION_TOKEN env var):
    # authenticate directly, persist to config, and return immediately.
    if not legacy:
        if not user_id:
            user_id = int(input("User ID (number): "))
        if not session_token:
            session_token = os.environ.get('SLCLI_SESSION_TOKEN', '') or input("Session Token: ")
        env.client.authenticate_with_hash(user_id, session_token)
        settings['access_token'] = session_token
        settings['userid'] = str(user_id)
        config_settings['softlayer'] = settings
        config.write_config(config_settings, env.config_file)
        click.echo(f"Logged in with session token for user ID {user_id}.")
        return

    username = settings.get('username') or os.environ.get('SLCLI_USER', None)
    password = os.environ.get('SLCLI_PASSWORD', '')
    yubi = None

    # Might already be logged in, try and refresh token
    if settings.get('access_token') and settings.get('userid'):
        env.client.authenticate_with_hash(settings.get('userid'), settings.get('access_token'))
        try:
            emp_id = settings.get('userid')
            env.client.call('SoftLayer_User_Employee', 'getObject', id=emp_id, mask="mask[id,username]")
            env.client.refresh_token(emp_id, settings.get('access_token'))
            env.client.call('SoftLayer_User_Employee', 'refreshEncryptedToken', settings.get('access_token'), id=emp_id)

            config_settings['softlayer'] = settings
            config.write_config(config_settings, env.config_file)
            return
        # pylint: disable=broad-exception-caught
        except Exception as ex:
            print("Error with Hash Authentication, try with password: {}".format(ex))

    url = settings.get('endpoint_url')
    click.echo("URL: {}".format(url))
    if username is None:
        username = input("Username: ")
    if not password:
        password = env.getpass("Password: ", default="")
    yubi = input("Yubi: ")

    try:
        result = env.client.authenticate_with_internal(username, password, str(yubi))
        print(result)
    # pylint: disable=broad-exception-caught
    except Exception as e:
        click.echo("EXCEPTION: {}".format(e))

    print("OK")



"""Removes unused Tags"""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.managers.tags import TagManager


@click.command(cls=SLCommand)
@click.option('--dry-run', '-d', is_flag=True, default=False,
              help="Don't delete, just show what will be deleted.")
@environment.pass_env
def cli(env, dry_run):
    """Removes all empty tags."""

    tag_manager = TagManager(env.client)
    empty_tags = tag_manager.get_unattached_tags()

    for tag in empty_tags:
        if dry_run:
            click.secho(f"(Dry Run) Removing {tag.get('name')}", fg='yellow')
        else:
            result = tag_manager.delete_tag(tag.get('name'))
            color = 'green' if result else 'red'
            click.secho(f"Removing {tag.get('name')}", fg=color)



"""List Reserved Capacity"""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer.CLI.virt.create import _parse_create_args as _parse_create_args
from SoftLayer.CLI.virt.create import _update_with_like_args as _update_with_like_args
from SoftLayer.managers.vs_capacity import CapacityManager as CapacityManager


@click.command(cls=SLCommand)
@click.option('--capacity-id', type=click.INT, help="Reserve capacity Id to provision this guest into.")
@click.option('--primary-disk', type=click.Choice(['25', '100']), default='25', help="Size of the main drive.")
@click.option('--hostname', '-H', required=True, prompt=True, help="Host portion of the FQDN.")
@click.option('--domain', '-D', required=True, prompt=True, help="Domain portion of the FQDN.")
@click.option('--os', '-o', help="OS install code. Tip: you can specify <OS>_LATEST.")
@click.option('--image', help="Image ID. See: 'slcli image list' for reference.")
@click.option('--boot-mode', type=click.STRING,
              help="Specify the mode to boot the OS in. Supported modes are HVM and PV.")
@click.option('--postinstall', '-i', help="Post-install script to download.")
@helpers.multi_option('--key', '-k', help="SSH keys to add to the root user.")
@helpers.multi_option('--disk', help="Additional disk sizes.")
@click.option('--private', is_flag=True, help="Forces the VS to only have access the private network.")
@click.option('--like', is_eager=True, callback=_update_with_like_args,
              help="Use the configuration from an existing VS.")
@click.option('--network', '-n', help="Network port speed in Mbps.")
@helpers.multi_option('--tag', '-g', help="Tags to add to the instance.")
@click.option('--userdata', '-u', help="User defined metadata string.")
@click.option('--ipv6', is_flag=True, help="Adds an IPv6 address to this guest")
@click.option('--test', is_flag=True,
              help="Test order, will return the order container, but not actually order a server.")
@environment.pass_env
def cli(env, **args):
    """Allows for creating a virtual guest in a reserved capacity. Only MONTHLY guests are supported at this time.

    If you would like support for hourly reserved capacity guests, please open an issue on the softlayer-python github.
    """
    create_args = _parse_create_args(env.client, args)

    create_args['primary_disk'] = args.get('primary_disk')
    manager = CapacityManager(env.client)
    capacity_id = args.get('capacity_id')
    test = args.get('test')

    result = manager.create_guest(capacity_id, test, create_args)

    env.fout(_build_receipt(result, test))


def _build_receipt(result, test=False):
    title = f"OrderId: {result.get('orderId', 'No order placed')}"
    table = formatting.Table(['Item Id', 'Description'], title=title)
    table.align['Description'] = 'l'

    if test:
        prices = result['prices']
    else:
        prices = result['orderDetails']['prices']

    for item in prices:
        table.add_row([item['id'], item['item']['description']])
    return table



"""Options for ordering a dedicated host"""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer

from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--datacenter', '-d',
              help="Router hostname (requires --flavor) "
                   "ex. ams01",
              show_default=True)
@click.option('--flavor', '-f',
              help="Dedicated Virtual Host flavor (requires --datacenter)"
                   " ex. 56_CORES_X_242_RAM_X_1_4_TB",
              show_default=True)
@environment.pass_env
def cli(env, **kwargs):
    """host order options for a given dedicated host.

    Example::
        To get a list of available backend routers see example:
        slcli dh create-options --datacenter dal05 --flavor 56_CORES_X_242_RAM_X_1_4_TB
"""

    mgr = SoftLayer.DedicatedHostManager(env.client)
    tables = []

    if not kwargs['flavor'] and not kwargs['datacenter']:
        options = mgr.get_create_options()

        # Datacenters
        dc_table = formatting.Table(['datacenter', 'value'])
        dc_table.sortby = 'value'
        for location in options['locations']:
            dc_table.add_row([location['name'], location['key']])
        tables.append(dc_table)

        dh_table = formatting.Table(['Dedicated Virtual Host Flavor(s)', 'value'])
        dh_table.sortby = 'value'
        for item in options['dedicated_host']:
            dh_table.add_row([item['name'], item['key']])
        tables.append(dh_table)
    else:
        if kwargs['flavor'] is None or kwargs['datacenter'] is None:
            raise exceptions.ArgumentError('Both a flavor and datacenter need '
                                           'to be passed as arguments '
                                           'ex. slcli dh create-options -d '
                                           'ams01 -f '
                                           '56_CORES_X_242_RAM_X_1_4_TB')
        router_opt = mgr.get_router_options(kwargs['datacenter'], kwargs['flavor'])
        br_table = formatting.Table(
            ['Available Backend Routers'])
        for router in router_opt:
            br_table.add_row([router['hostname']])
        tables.append(br_table)

    env.fout(tables)



credentialCreate = {
    "accountId": "12345",
    "createDate": "2019-04-05T13:25:25-06:00",
    "id": 11111,
    "password": "nwUEUsx6PiEoN0B1Xe9z9hUCyXMkAFhDOjHqYJva",
    "username": "XfHhBNBPlPdlWyaPPJAI",
    "type": {
        "description": "A credential for generating S3 Compatible Signatures.",
        "keyName": "S3_COMPATIBLE_SIGNATURE",
        "name": "S3 Compatible Signature"
    }
}

getCredentials = [
    {
        "accountId": "12345",
        "createDate": "2019-04-05T13:25:25-06:00",
        "id": 11111,
        "password": "nwUEUsx6PiEoN0B1Xe9z9hUCyXMkAFhDOjHqYJva",
        "username": "XfHhBNBPlPdlWyaPPJAI",
        "type": {
            "description": "A credential for generating S3 Compatible Signatures.",
            "keyName": "S3_COMPATIBLE_SIGNATURE",
            "name": "S3 Compatible Signature"
        }
    },
    {
        "accountId": "12345",
        "createDate": "2019-04-05T13:25:25-06:00",
        "id": 11111,
        "password": "nwUEUsx6PiEoN0B1Xe9z9hUCyXMkAFhDOjHqYJva",
        "username": "XfHhBNBPlPdlWyaPPJAI",
        "type": {
            "description": "A credential for generating S3 Compatible Signatures.",
            "keyName": "S3_COMPATIBLE_SIGNATURE",
            "name": "S3 Compatible Signature"
        }
    }
]

getBuckets = [
    {
        "bytesUsed": 40540117,
        "name": "normal-bucket",
        "objectCount": 4,
        "storageLocation": "us-standard"
    }
]

getEndpoints = [
    {
        'legacy': False,
        'region': 'us-geo',
        'type': 'public',
        'url': 's3.us.cloud-object-storage.appdomain.cloud'
    },
    {
        'legacy': False,
        'region': 'us-geo',
        'type': 'private',
        'url': 's3.private.us.cloud-object-storage.appdomain.cloud'
    }
]
getCredentialLimit = 2

credentialDelete = True

getObject = {
    'id': 123456,
    'username': 'TEST307608-1',
    'credentials': [
        {
            'id': 1933496,
            'password': 'Um1Bp420FIFNvAg2QHjn5Sci2c2x4RNDXpVDDvnfsdsd1010',
            'username': 'Kv9aNIhtNa7ZRceabecs',
            'type': {
                'description': 'A credential for generating S3 Compatible Signatures.',
                'keyName': 'S3_COMPATIBLE_SIGNATURE'
            }
        },
        {

            'id': 1732820,
            'password': 'q6NtwqeuXDaRqGc0Jrugg2sDgbatyNsoN9sPEmjo',
            'username': '252r9BN8ibuDSQAXLOeL',
            'type': {
                'description': 'A credential for generating S3 Compatible Signatures.',
                'keyName': 'S3_COMPATIBLE_SIGNATURE',
            }
        }
    ],
    'uuid': '01c449c484ae4a58a42d9b79d4c5e4ed'
}



"""List packages."""
# :license: MIT, see LICENSE for more details.
import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers import network
from SoftLayer.managers import ordering

COLUMNS = ['id', 'dc', 'description', 'keyName', 'Note']


@click.command(cls=SLCommand)
@click.argument('package_keyname')
@environment.pass_env
def cli(env, package_keyname):
    """List Datacenters a package can be ordered in.

    Use the location Key Name to place orders
    """
    manager = ordering.OrderingManager(env.client)
    network_manager = network.NetworkManager(env.client)

    pods = network_manager.get_closed_pods()
    table = formatting.Table(COLUMNS)

    locations = manager.package_locations(package_keyname)
    for region in locations:
        for datacenter in region['locations']:
            closure = []
            for pod in pods:
                if datacenter['location']['name'] in str(pod['name']):
                    closure.append(pod['name'])

            notes = '-'
            if len(closure) > 0:
                notes = 'closed soon: %s' % (', '.join(closure))
            table.add_row([
                datacenter['location']['id'],
                datacenter['location']['name'],
                region['description'],
                region['keyname'], notes
            ])
    env.fout(table)



"""Remove SSL certificate."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """Remove SSL certificate."""

    manager = SoftLayer.SSLManager(env.client)
    if not (env.skip_confirmations or formatting.no_going_back('yes')):
        raise exceptions.CLIAbort("Aborted.")

    manager.remove_certificate(identifier)



"""Order a file storage volume."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting


CONTEXT_SETTINGS = {'token_normalize_func': lambda x: x.upper()}


@click.command(cls=SoftLayer.CLI.command.SLCommand, context_settings=CONTEXT_SETTINGS)
@click.option('--storage-type', required=True, type=click.Choice(['performance', 'endurance']),
              help='Type of file storage volume')
@click.option('--size', type=int, required=True,
              help='Size of file storage volume in GB')
@click.option('--iops', type=int,
              help="""Performance Storage IOPs. Options vary based on storage size.
[required for storage-type performance]""")
@click.option('--tier', type=click.Choice(['0.25', '2', '4', '10']),
              help='Endurance Storage Tier (IOP per GB) [required for storage-type endurance]')
@click.option('-l', '--location', required=True,
              help='Datacenter short name (e.g.: dal09)')
@click.option('--snapshot-size', type=int,
              help='Optional parameter for ordering snapshot '
              'space along with endurance file storage; specifies '
              'the size (in GB) of snapshot space to order')
@click.option('--service-offering',
              help="""The service offering package to use for placing the order.
[optional, default is \'storage_as_a_service\']. enterprise and performance are depreciated""",
              default='storage_as_a_service',
              type=click.Choice([
                  'storage_as_a_service',
                  'enterprise',
                  'performance']))
@click.option('--billing', type=click.Choice(['hourly', 'monthly']), default='monthly',
              help="Optional parameter for Billing rate (default to monthly)")
@click.option('--force', default=False, is_flag=True, help="Force order file storage volume without confirmation")
@environment.pass_env
def cli(env, storage_type, size, iops, tier,
        location, snapshot_size, service_offering, billing, force):
    """Order a file storage volume.

    Example::
        slcli file volume-order --storage-type performance --size 1000 --iops 4000 -d dal09
        This command orders a performance volume with size is 1000GB, IOPS is 4000, located at dal09.

        slcli file volume-order --storage-type endurance --size 500 --tier 4 -d dal09 --snapshot-size 500
        This command orders an endurance volume with size is 500GB, tier level is 4 IOPS per GB,\
located at dal09 with an additional snapshot space size of 500GB

    Valid size and iops options can be found here:
    https://cloud.ibm.com/docs/FileStorage/index.html#provisioning-considerations
    """
    file_manager = SoftLayer.FileStorageManager(env.client)
    storage_type = storage_type.lower()

    if not force:
        if not (env.skip_confirmations or
                formatting.confirm("This action will incur charges on your account. Continue?")):
            raise exceptions.CLIAbort('Aborted')

    hourly_billing_flag = False
    if billing.lower() == "hourly":
        hourly_billing_flag = True

    if service_offering != 'storage_as_a_service':
        click.secho(f'{service_offering} is a legacy storage offering', fg='red')
        if hourly_billing_flag:
            raise exceptions.CLIAbort(
                'Hourly billing is only available for the storage_as_a_service service offering'
            )

    order = {}
    if storage_type == 'performance':
        if iops is None:
            raise exceptions.CLIAbort('Option --iops required with Performance')

        if service_offering == 'performance' and snapshot_size is not None:
            raise exceptions.CLIAbort(
                '--snapshot-size is not available for performance service offerings. '
                'Use --service-offering storage_as_a_service'
            )

        try:
            order = file_manager.order_file_volume(
                storage_type=storage_type,
                location=location,
                size=size,
                iops=iops,
                snapshot_size=snapshot_size,
                service_offering=service_offering,
                hourly_billing_flag=hourly_billing_flag
            )
        except ValueError as ex:
            raise exceptions.ArgumentError(str(ex))

    if storage_type == 'endurance':
        if tier is None:
            raise exceptions.CLIAbort(
                'Option --tier required with Endurance in IOPS/GB [0.25,2,4,10]'
            )

        try:
            order = file_manager.order_file_volume(
                storage_type=storage_type,
                location=location,
                size=size,
                tier_level=float(tier),
                snapshot_size=snapshot_size,
                service_offering=service_offering,
                hourly_billing_flag=hourly_billing_flag
            )
        except ValueError as ex:
            raise exceptions.ArgumentError(str(ex))

    if 'placedOrder' in order.keys():
        click.echo(f"Order #{order['placedOrder']['id']} placed successfully!")
        for item in order['placedOrder']['items']:
            click.echo(" > %s" % item['description'])
        click.echo(
            f"\nYou may run \"slcli file volume-list --order {order['placedOrder']['id']}\" to find this file "
            "volume after it is ready.")
    else:
        click.echo("Order could not be placed! Please verify your options and try again.")



# pylint: skip-file
HARDWARE_ITEMS = [
    {'attributes': [],
     'capacity': '999',
     'description': 'Unknown',
     'itemCategory': {'categoryCode': 'unknown', 'id': 325},
     'keyName': 'UNKNOWN',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 1245172,
                 "locationGroupId": '',
                 'itemId': 935954,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0',
                 'setupFee': '0',
                 'sort': 0}]},
    {'attributes': [],
     'capacity': '64',
     'description': '1 IPv6 Address',
     'itemCategory': {'categoryCode': 'pri_ipv6_addresses',
                      'id': 325},
     'keyName': '1_IPV6_ADDRESS',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 17129,
                 "locationGroupId": '',
                 'itemId': 4097,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0',
                 'setupFee': '0',
                 'sort': 0}]},
    {'attributes': [],
     'capacity': '10',
     'description': '10 Mbps Public & Private Network Uplinks',
     'itemCategory': {'categoryCode': 'port_speed', 'id': 26},
     'keyName': '10_MBPS_PUBLIC_PRIVATE_NETWORK_UPLINKS',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 272,
                 "locationGroupId": '',
                 'itemId': 186,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0',
                 'setupFee': '0',
                 'sort': 5}]},
    {'attributes': [],
     'capacity': '0',
     'description': 'Ubuntu Linux 14.04 LTS Trusty Tahr (64 bit)',
     'itemCategory': {'categoryCode': 'os', 'id': 12},
     'keyName': 'OS_UBUNTU_14_04_LTS_TRUSTY_TAHR_64_BIT',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 37650,
                 "locationGroupId": '',
                 'itemId': 4702,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0',
                 'setupFee': '0',
                 'sort': 9}],
     'softwareDescription': {'id': 1362,
                             'longDescription': 'Ubuntu / 14.04-64',
                             'referenceCode': 'UBUNTU_14_64'}},
    {'attributes': [],
     'capacity': '1',
     'description': '1 IP Address',
     'itemCategory': {'categoryCode': 'pri_ip_addresses', 'id': 13},
     'keyName': '1_IP_ADDRESS',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 21,
                 "locationGroupId": '',
                 'itemId': 15,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0',
                 'setupFee': '0',
                 'sort': 0}]},
    {'attributes': [{'attributeTypeKeyName': 'RECLAIM_BYPASS',
                     'id': 1014}],
     'description': 'Unlimited SSL VPN Users',
     'itemCategory': {'categoryCode': 'vpn_management', 'id': 31},
     'keyName': 'SSL_VPN_USERS_1_PPTP_VPN_USER_PER_ACCOUNT',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 420,
                 "locationGroupId": '',
                 'itemId': 309,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0',
                 'setupFee': '0',
                 'sort': 0}]},
    {'attributes': [],
     'description': 'Reboot / KVM over IP',
     'itemCategory': {'categoryCode': 'remote_management',
                      'id': 46},
     'keyName': 'REBOOT_KVM_OVER_IP',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 906,
                 "locationGroupId": '',
                 'itemId': 504,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0',
                 'setupFee': '0',
                 'sort': 0}]},
    {'attributes': [],
     'capacity': '0',
     'description': '0 GB Bandwidth',
     'itemCategory': {'categoryCode': 'bandwidth', 'id': 10},
     'keyName': 'BANDWIDTH_0_GB',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'id': 22505,
                 "locationGroupId": '',
                 'itemId': 4481,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0',
                 'setupFee': '0',
                 'sort': 98}]},
    {'attributes': [],
     'capacity': '0',
     'description': '0 GB Bandwidth',
     'itemCategory': {'categoryCode': 'bandwidth', 'id': 10},
     'keyName': 'BANDWIDTH_0_GB_2',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 1800,
                 "locationGroupId": '',
                 'itemId': 439,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'setupFee': '0',
                 'sort': 99}]}]

ENTERPRISE_PACKAGE = {
    'categories': [
        {'categoryCode': 'storage_service_enterprise'}
    ],
    'id': 240,
    'name': 'Endurance',
    'items': [
        {
            'capacity': '0',
            'itemCategory': {'categoryCode': 'storage_service_enterprise'},
            'keyName': 'CODENAME_PRIME_STORAGE_SERVICE',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'storage_service_enterprise'}
                    ],
                    'id': 45058,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'itemCategory': {'categoryCode': 'storage_file'},
            'keyName': 'FILE_STORAGE_2',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'storage_file'}
                    ],
                    'id': 45108,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'itemCategory': {'categoryCode': 'storage_block'},
            'keyName': 'BLOCK_STORAGE_2',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'storage_block'}
                    ],
                    'id': 45098,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '10',
            'itemCategory': {'categoryCode': 'performance_storage_space'},
            'keyName': '10_GB_STORAGE_SPACE',
            'prices': [
                {
                    'capacityRestrictionMaximum': '200',
                    'capacityRestrictionMinimum': '200',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'storage_snapshot_space'}
                    ],
                    'id': 46160,
                    'locationGroupId': ''
                }, {
                    'capacityRestrictionMaximum': '300',
                    'capacityRestrictionMinimum': '300',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'storage_snapshot_space'}
                    ],
                    'id': 46170,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '20',
            'itemCategory': {'categoryCode': 'performance_storage_space'},
            'keyName': '20_GB_PERFORMANCE_STORAGE_SPACE',
            'prices': [
                {
                    'capacityRestrictionMaximum': '200',
                    'capacityRestrictionMinimum': '200',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'storage_snapshot_space'}
                    ],
                    'id': 45860,
                    'locationGroupId': ''
                }, {
                    'capacityRestrictionMaximum': '200',
                    'capacityRestrictionMinimum': '200',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'performance_storage_replication'}
                    ],
                    'id': 46659,
                    'locationGroupId': ''
                }, {
                    'capacityRestrictionMaximum': '200',
                    'capacityRestrictionMinimum': '200',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'performance_storage_space'}
                    ],
                    'id': 45128,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '1000',
            'itemCategory': {'categoryCode': 'performance_storage_space'},
            'keyName': '1000_GB_PERFORMANCE_STORAGE_SPACE',
            'prices': [
                {
                    'capacityRestrictionMaximum': '300',
                    'capacityRestrictionMinimum': '300',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'performance_storage_replication'}
                    ],
                    'id': 46789,
                    'locationGroupId': ''
                }, {
                    'capacityRestrictionMaximum': '300',
                    'capacityRestrictionMinimum': '300',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'performance_storage_space'}
                    ],
                    'id': 45318,
                    'locationGroupId': ''
                }
            ]
        }, {
            'attributes': [
                {'value': '300'}
            ],
            'capacity': '300',
            'itemCategory': {'categoryCode': 'storage_tier_level'},
            'keyName': 'WRITEHEAVY_TIER',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'storage_tier_level'}
                    ],
                    'id': 45088,
                    'locationGroupId': ''
                }
            ]
        }, {
            'attributes': [
                {'value': '200'}
            ],
            'capacity': '200',
            'itemCategory': {'categoryCode': 'storage_tier_level'},
            'keyName': 'READHEAVY_TIER',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'storage_tier_level'}
                    ],
                    'id': 45078,
                    'locationGroupId': ''
                }
            ]
        }
    ]
}

PERFORMANCE_PACKAGE = {
    'categories': [
        {'categoryCode': 'performance_storage_iscsi'},
        {'categoryCode': 'performance_storage_nfs'}
    ],
    'id': 222,
    'name': 'Performance',
    'items': [
        {
            'capacity': '0',
            'itemCategory': {'categoryCode': 'performance_storage_iscsi'},
            'keyName': 'BLOCK_STORAGE_PERFORMANCE_ISCSI',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'performance_storage_iscsi'}
                    ],
                    'id': 40672,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'itemCategory': {'categoryCode': 'performance_storage_nfs'},
            'keyName': 'FILE_STORAGE_PERFORMANCE_NFS',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'performance_storage_nfs'}
                    ],
                    'id': 40662,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '20',
            'itemCategory': {'categoryCode': 'performance_storage_space'},
            'keyName': '20_GB_PERFORMANCE_STORAGE_SPACE',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'performance_storage_space'}
                    ],
                    'id': 40682,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '1000',
            'itemCategory': {'categoryCode': 'performance_storage_space'},
            'keyName': '1000_GB_PERFORMANCE_STORAGE_SPACE',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'performance_storage_space'}
                    ],
                    'id': 40742,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '800',
            'itemCategory': {'categoryCode': 'performance_storage_iops'},
            'keyName': '800_IOPS_4',
            'prices': [
                {
                    'capacityRestrictionMaximum': '1000',
                    'capacityRestrictionMinimum': '100',
                    'capacityRestrictionType': 'STORAGE_SPACE',
                    'categories': [
                        {'categoryCode': 'performance_storage_iops'}
                    ],
                    'id': 41562,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '1000',
            'itemCategory': {'categoryCode': 'performance_storage_iops'},
            'keyName': '1000_IOPS',
            'prices': [
                {
                    'capacityRestrictionMaximum': '20',
                    'capacityRestrictionMinimum': '20',
                    'capacityRestrictionType': 'STORAGE_SPACE',
                    'categories': [
                        {'categoryCode': 'performance_storage_iops'}
                    ],
                    'id': 40882,
                    'locationGroupId': ''
                }
            ]
        }
    ]
}

SAAS_PACKAGE = {
    'categories': [
        {'categoryCode': 'storage_as_a_service'}
    ],
    'id': 759,
    'name': 'Storage As A Service (StaaS)',
    'items': [
        {
            'capacity': '0',
            'keyName': '',
            'prices': [
                {
                    'id': 189433,
                    'categories': [
                        {'categoryCode': 'storage_as_a_service'}
                    ],
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'keyName': '',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'storage_block'}
                    ],
                    'id': 189443,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'keyName': '',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'storage_file'}
                    ],
                    'id': 189453,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'capacityMaximum': '999',
            'capacityMinimum': '500',
            'itemCategory': {'categoryCode': 'performance_storage_space'},
            'keyName': '500_999_GBS',
            'prices': [
                {
                    'id': 189993,
                    'categories': [
                        {'categoryCode': 'performance_storage_space'}
                    ],
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'capacityMaximum': '1999',
            'capacityMinimum': '1000',
            'itemCategory': {'categoryCode': 'performance_storage_space'},
            'keyName': '1000_1999_GBS',
            'prices': [
                {
                    'id': 190113,
                    'categories': [
                        {'categoryCode': 'performance_storage_space'}
                    ],
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'capacityMaximum': '12000',
            'capacityMinimum': '1',
            'keyName': 'STORAGE_SPACE_FOR_2_IOPS_PER_GB',
            'prices': [
                {
                    'id': 193433,
                    'categories': [
                        {'categoryCode': 'performance_storage_space'}
                    ],
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'capacityMaximum': '12000',
            'capacityMinimum': '1',
            'keyName': 'STORAGE_SPACE_FOR_4_IOPS_PER_GB',
            'prices': [
                {
                    'id': 194763,
                    'categories': [
                        {'categoryCode': 'performance_storage_space'}
                    ],
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'capacityMaximum': '10000',
            'capacityMinimum': '100',
            'keyName': '',
            'itemCategory': {'categoryCode': 'performance_storage_iops'},
            'prices': [
                {
                    'capacityRestrictionMaximum': '999',
                    'capacityRestrictionMinimum': '500',
                    'capacityRestrictionType': 'STORAGE_SPACE',
                    'categories': [
                        {'categoryCode': 'performance_storage_iops'}
                    ],
                    'id': 190053,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'capacityMaximum': '20000',
            'capacityMinimum': '100',
            'keyName': '',
            'itemCategory': {'categoryCode': 'performance_storage_iops'},
            'prices': [
                {
                    'capacityRestrictionMaximum': '1999',
                    'capacityRestrictionMinimum': '1000',
                    'capacityRestrictionType': 'STORAGE_SPACE',
                    'categories': [
                        {'categoryCode': 'performance_storage_iops'}
                    ],
                    'id': 190173,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '200',
            'itemCategory': {'categoryCode': 'storage_tier_level'},
            'keyName': '',
            'prices': [
                {
                    'id': 193373,
                    'categories': [
                        {'categoryCode': 'storage_tier_level'}
                    ],
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '300',
            'itemCategory': {'categoryCode': 'storage_tier_level'},
            'keyName': '',
            'prices': [
                {
                    'id': 194703,
                    'categories': [
                        {'categoryCode': 'storage_tier_level'}
                    ],
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '10',
            'keyName': '',
            'prices': [
                {
                    'capacityRestrictionMaximum': '48000',
                    'capacityRestrictionMinimum': '100',
                    'capacityRestrictionType': 'IOPS',
                    'categories': [
                        {'categoryCode': 'storage_snapshot_space'}
                    ],
                    'id': 191193,
                    'locationGroupId': ''
                }, {
                    'capacityRestrictionMaximum': '200',
                    'capacityRestrictionMinimum': '200',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'storage_snapshot_space'}
                    ],
                    'id': 193613,
                    'locationGroupId': ''
                }, {
                    'capacityRestrictionMaximum': '300',
                    'capacityRestrictionMinimum': '300',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'storage_snapshot_space'}
                    ],
                    'id': 194943,
                    'locationGroupId': ''}]
        }, {
            'capacity': '20',
            'keyName': '',
            'prices': [
                {
                    'capacityRestrictionMaximum': '200',
                    'capacityRestrictionMinimum': '200',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'storage_snapshot_space'}
                    ],
                    'id': 193853,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'itemCategory': {
                'categoryCode': 'performance_storage_replication'
            },
            'keyName': 'REPLICATION_FOR_IOPSBASED_PERFORMANCE',
            'prices': [
                {
                    'capacityRestrictionMaximum': '48000',
                    'capacityRestrictionMinimum': '1',
                    'capacityRestrictionType': 'IOPS',
                    'categories': [
                        {'categoryCode': 'performance_storage_replication'}
                    ],
                    'id': 192033,
                    'locationGroupId': ''
                }
            ]
        }, {
            'capacity': '0',
            'itemCategory': {
                'categoryCode': 'performance_storage_replication'
            },
            'keyName': 'REPLICATION_FOR_TIERBASED_PERFORMANCE',
            'prices': [
                {
                    'capacityRestrictionMaximum': '200',
                    'capacityRestrictionMinimum': '200',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'performance_storage_replication'}
                    ],
                    'id': 194693,
                    'locationGroupId': ''
                }
            ]
        }
    ]
}

SAAS_REST_PACKAGE = {
    'categories': [
        {'categoryCode': 'storage_as_a_service'}
    ],
    'id': 759,
    'name': 'Storage As A Service (StaaS)',
    'items': [
        {
            'capacity': '0',
            'keyName': '',
            'prices': [
                {
                    'id': 189433,
                    'categories': [
                        {'categoryCode': 'storage_as_a_service'}
                    ],
                    'locationGroupId': None
                }
            ]
        }, {
            'capacity': '20',
            'keyName': '',
            'prices': [
                {
                    'capacityRestrictionMaximum': '200',
                    'capacityRestrictionMinimum': '200',
                    'capacityRestrictionType': 'STORAGE_TIER_LEVEL',
                    'categories': [
                        {'categoryCode': 'storage_snapshot_space'}
                    ],
                    'id': 193853,
                    'locationGroupId': None
                }
            ]
        }, {
            'capacity': '0',
            'capacityMaximum': '1999',
            'capacityMinimum': '1000',
            'itemCategory': {'categoryCode': 'performance_storage_space'},
            'keyName': '1000_1999_GBS',
            'prices': [
                {
                    'id': 190113,
                    'categories': [
                        {'categoryCode': 'performance_storage_space'}
                    ],
                    'locationGroupId': None
                }
            ]
        }, {
            'capacity': '0',
            'capacityMaximum': '20000',
            'capacityMinimum': '100',
            'keyName': '',
            'itemCategory': {'categoryCode': 'performance_storage_iops'},
            'prices': [
                {
                    'capacityRestrictionMaximum': '1999',
                    'capacityRestrictionMinimum': '1000',
                    'capacityRestrictionType': 'STORAGE_SPACE',
                    'categories': [
                        {'categoryCode': 'performance_storage_iops'}
                    ],
                    'id': 190173,
                    'locationGroupId': None
                }
            ]
        }, {
            'capacity': '0',
            'keyName': '',
            'prices': [
                {
                    'categories': [
                        {'categoryCode': 'storage_file'}
                    ],
                    'id': 189453,
                    'locationGroupId': None
                }
            ]
        }
    ]
}

activePreset1 = {
    'description': 'Single Xeon 1270, 8GB Ram, 2x1TB SATA disks, Non-RAID',
    'id': 64,
    'isActive': '1',
    'keyName': 'S1270_8GB_2X1TBSATA_NORAID',
    'name': 'S1270 8GB 2X1TBSATA NORAID',
    'packageId': 200,
    'prices': [
        {
            "hourlyRecurringFee": "1.18",
            "id": 165711,
            "locationGroupId": '',
            "recurringFee": "780",
        }
    ]
}

activePreset2 = {
    'description': 'Dual Xeon Gold, 384GB Ram, 4x960GB SSD, RAID 10',
    'id': 65,
    'isActive': '1',
    'keyName': 'DGOLD_6140_384GB_4X960GB_SSD_SED_RAID_10',
    'name': 'DGOLD 6140 384GB 4X960GB SSD SED RAID 10',
    'packageId': 200,
    'prices': [
        {
            "hourlyRecurringFee": "1.18",
            "id": 165711,
            "locationGroupId": '',
            "recurringFee": "780",
        }
    ]
}

getAllObjects = [{
    'activePresets': [activePreset1],
    'accountRestrictedActivePresets': [activePreset2],
    'description': 'Bare Metal Server',
    'firstOrderStepId': 1,
    'id': 200,
    'isActive': 1,
    'items': HARDWARE_ITEMS,
    'name': 'Bare Metal Server',
    'regions': [{'description': 'WDC01 - Washington, DC - East Coast U.S.',
                 'keyname': 'WASHINGTON_DC',
                 'location': {'location': {'id': 37473,
                                           'longName': 'Washington 1',
                                           'name': 'wdc01'}},
                 'sortOrder': 10}],
    'subDescription': 'Bare Metal Server',
    'unitSize': 1,
    "itemPrices": [
        {
            "hourlyRecurringFee": ".027",
            "id": 205911,
            "laborFee": "0",
            "locationGroupId": 505,
            "capacityRestrictionMaximum": "40",
            "capacityRestrictionMinimum": "40",
            "capacityRestrictionType": "CORE",
            "item": {
                "capacity": "0",
                "description": "Load Balancer Uptime",
                "id": 10785,
                "keyName": "LOAD_BALANCER_UPTIME",
            }
        },
        {
            "hourlyRecurringFee": "0",
            "id": 199467,
            "laborFee": "0",
            "locationGroupId": '',
            "recurringFee": "0",
            "item": {
                "capacity": "0",
                "description": "Load Balancer Bandwidth",
                "id": 10051,
                "keyName": "LOAD_BALANCER_BANDWIDTH",
            }
        },
        {
            "hourlyRecurringFee": ".028",
            "id": 205913,
            "laborFee": "0",
            "locationGroupId": 507,
            "item": {
                "capacity": "0",
                "description": "Load Balancer Uptime",
                "id": 10785,
                "keyName": "LOAD_BALANCER_UPTIME",
            }
        }]
}]

getItems = [
    {
        'id': 1234,
        'keyName': 'KeyName01',
        'capacity': '1000',
        'description': 'Public & Private Networks',
        'itemCategory': {'categoryCode': 'Uplink Port Speeds'},
        'softwareDescription': {
            'id': 1228,
            'longDescription': 'Redhat EL 5.10-64',
            'referenceCode': 'REDHAT_5_64'
        },
        'prices': [{'id': 1122,
                    'hourlyRecurringFee': 0.10,
                    'recurringFee': 0.10,
                    'categories': [{'id': 26,
                                    'name': 'Uplink Port Speeds',
                                    'categoryCode': 'port_speed'}]}],
    },
    {
        'id': 2233,
        'keyName': 'KeyName02',
        'capacity': '1000',
        'description': 'Public & Private Networks',
        'itemCategory': {'categoryCode': 'Uplink Port Speeds'},
        'prices': [{'id': 4477,
                    'hourlyRecurringFee': 0.10,
                    'recurringFee': 0.10,
                    'categories': [{'id': 26,
                                    'name': 'Uplink Port Speeds',
                                    'categoryCode': 'port_speed'}]}],
    },
    {
        'id': 1239,
        'keyName': 'KeyName03',
        'capacity': '2',
        'description': 'RAM',
        'itemCategory': {'categoryCode': 'RAM'},
        'prices': [{'id': 1133,
                    'hourlyRecurringFee': 0.0,
                    'recurringFee': 0.0,
                    'categories': [{'id': 3,
                                    'name': 'RAM',
                                    'categoryCode': 'ram'}]}],
    },
    {
        'id': 1240,
        'keyName': 'KeyName014',
        'capacity': '4',
        'units': 'PRIVATE_CORE',
        'description': 'Computing Instance (Dedicated)',
        'itemCategory': {'categoryCode': 'Computing Instance'},
        'prices': [{'id': 1007,
                    'hourlyRecurringFee': 0.0,
                    'recurringFee': 0.0,
                    'categories': [{'id': 80,
                                    'name': 'Computing Instance',
                                    'categoryCode': 'guest_core'}]}],
    },
    {
        'id': 1250,
        'keyName': 'KeyName015',
        'capacity': '4',
        'units': 'CORE',
        'description': 'Computing Instance',
        'itemCategory': {'categoryCode': 'Computing Instance'},
        'prices': [{'id': 1144,
                    'locationGroupId': None,
                    'hourlyRecurringFee': 0.10,
                    'recurringFee': 0.10,
                    'categories': [{'id': 80,
                                    'name': 'Computing Instance',
                                    'categoryCode': 'guest_core'}]}],
    },
    {
        'id': 112233,
        'keyName': 'KeyName016',
        'capacity': '55',
        'units': 'CORE',
        'description': 'Computing Instance',
        'itemCategory': {'categoryCode': 'Computing Instance'},
        'prices': [{'id': 332211,
                    'locationGroupId': 1,
                    'hourlyRecurringFee': 0.0,
                    'recurringFee': 0.0,
                    'categories': [{'id': 80,
                                    'name': 'Computing Instance',
                                    'categoryCode': 'guest_core'}]}],
    },
    {
        'id': 4439,
        'keyName': 'KeyName017',
        'capacity': '1',
        'description': '1 GB iSCSI Storage',
        'itemCategory': {'categoryCode': 'iscsi'},
        'prices': [{'id': 2222, 'hourlyRecurringFee': 0.10, 'recurringFee': 0.10}],
    },
    {
        'id': 1121,
        'keyName': 'KeyName081',
        'capacity': '20',
        'description': '20 GB iSCSI snapshot',
        'itemCategory': {'categoryCode': 'iscsi_snapshot_space'},
        'prices': [{'id': 2014, 'hourlyRecurringFee': 0.10}],
    },
    {
        'id': 4440,
        'keyName': 'KeyName019',
        'capacity': '4',
        'description': '4 Portable Public IP Addresses',
        'itemCategory': {'categoryCode': 'sov_sec_ip_addresses_pub'},
        'prices': [{'id': 4444, 'hourlyRecurringFee': 0.10, 'recurringFee': 0.10}],
    },
    {
        'id': 8880,
        'keyName': 'KeyName0199',
        'capacity': '8',
        'description': '8 Portable Public IP Addresses',
        'itemCategory': {'categoryCode': 'sov_sec_ip_addresses_pub'},
        'prices': [{'id': 8888, 'hourlyRecurringFee': 0.10, 'recurringFee': 0.10}],
    },
    {
        'id': 44400,
        'keyName': 'KeyName0155',
        'capacity': '4',
        'description': '4 Portable Private IP Addresses',
        'itemCategory': {'categoryCode': 'sov_sec_ip_addresses_priv'},
        'prices': [{'id': 44441, 'hourlyRecurringFee': 0.10, 'recurringFee': 0.10}],
    },
    {
        'id': 88800,
        'keyName': 'KeyName0144',
        'capacity': '8',
        'description': '8 Portable Private IP Addresses',
        'itemCategory': {'categoryCode': 'sov_sec_ip_addresses_priv'},
        'prices': [{'id': 88881, 'hourlyRecurringFee': 0.0, 'recurringFee': 0.0}],
    },
    {
        'id': 10,
        'keyName': 'KeyName0341',
        'capacity': '0',
        'description': 'Global IPv4',
        'itemCategory': {'categoryCode': 'global_ipv4'},
        'prices': [{'id': 11, 'hourlyRecurringFee': 0.0, 'recurringFee': 0.0}],
    },
    {
        'id': 66464,
        'keyName': '1_IPV6_ADDRESS',
        'capacity': '64',
        'description': '/64 Block Portable Public IPv6 Addresses',
        'itemCategory': {'categoryCode': 'static_ipv6_addresses'},
        'prices': [{'id': 664641, 'hourlyRecurringFee': '0', 'locationGroupId': '', 'recurringFee': '0'}],
    },
    {
        'id': 610,
        'keyName': 'KeyName031',
        'capacity': '0',
        'description': 'Global IPv6',
        'itemCategory': {'categoryCode': 'global_ipv6'},
        'prices': [{'id': 611, 'hourlyRecurringFee': 0.10, 'recurringFee': 0.10}],
    },
    {'attributes': [],
     'capacity': '0',
     'description': '0 GB Bandwidth',
     'itemCategory': {'categoryCode': 'bandwidth', 'id': 10},
     'keyName': 'BANDWIDTH_0_GB_2',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 1800,
                 "locationGroupId": '',
                 'itemId': 439,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'setupFee': '0',
                 'sort': 99}]},
    {'attributes': [],
     'capacity': '10',
     'description': '10 Mbps Public & Private Network Uplinks',
     'itemCategory': {'categoryCode': 'port_speed', 'id': 26},
     'keyName': '10_MBPS_PUBLIC_PRIVATE_NETWORK_UPLINKS',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0',
                 'id': 272,
                 "locationGroupId": '',
                 'itemId': 186,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0',
                 'setupFee': '0',
                 'sort': 5}]},
    {'attributes': [],
     'capacity': '0',
     'description': 'Ubuntu Linux 14.04 LTS Trusty Tahr (64 bit)',
     'itemCategory': {'categoryCode': 'os', 'id': 12},
     'keyName': 'OS_UBUNTU_14_04_LTS_TRUSTY_TAHR_64_BIT',
     'prices': [{'accountRestrictions': [],
                 'currentPriceFlag': '',
                 'hourlyRecurringFee': '0.10',
                 'id': 37650,
                 "locationGroupId": '',
                 'itemId': 4702,
                 'laborFee': '0',
                 'onSaleFlag': '',
                 'oneTimeFee': '0',
                 'quantity': '',
                 'recurringFee': '0.1',
                 'setupFee': '0.1',
                 'sort': 9}],
     'softwareDescription': {'id': 1362,
                             'longDescription': 'Ubuntu / 14.04-64',
                             'referenceCode': 'UBUNTU_14_64'}}
]

getItemPricesISCSI = [
    {
        'currentPriceFlag': '',
        'id': 2152,
        'item': {
            'capacity': '1',
            'description': '1 GB iSCSI SAN Storage',
            'id': 1111,
            'softwareDescriptionId': '',
            'units': 'GB',
            'upgradeItemId': 547},
        'itemId': 1111,
        'laborFee': '0',
        'onSaleFlag': '',
        'oneTimeFee': '0',
        'packageReferences': [{'id': 46626,
                               'itemPriceId': 2152, 'packageId': 0}],
        'quantity': '',
        'recurringFee': '.35',
        'setupFee': '0',
        'sort': 0
    },
    {
        'currentPriceFlag': '',
        'id': 22501,
        'item': {'capacity': '1',
                 'description': '1 GB iSCSI SAN Storage',
                 'id': 1111,
                 'softwareDescriptionId': '',
                 'units': 'GB',
                 'upgradeItemId': 547},
        'itemId': 1111,
        'laborFee': '0',
        'onSaleFlag': '',
        'oneTimeFee': '0',
        'packageReferences': [{
            'id': 252983,
            'itemPriceId': 22501, 'packageId': 0
        }],
        'quantity': '',
        'recurringFee': '0',
        'setupFee': '0',
        'sort': 0
    },
    {
        'currentPriceFlag': '',
        'id': 22441,
        'item': {
            'capacity': '1',
            'description': '1 GB iSCSI SAN Storage',
            'id': 1111,
            'softwareDescriptionId': '',
            'units': 'GB',
            'upgradeItemId': 547
        },
        'itemId': 1111,
        'laborFee': '0',
        'onSaleFlag': '',
        'oneTimeFee': '0',
        'packageReferences': [{'id': 250326,
                               'itemPriceId': 22441, 'packageId': 0}],
        'quantity': '',
        'recurringFee': '15',
        'setupFee': '0',
        'sort': 0
    }]

getItemsVS = [
    {
        'id': 1234,
        'keyName': 'KeyName01',
        'capacity': '1000',
        'description': 'Public & Private Networks',
        'itemCategory': {'categoryCode': 'Uplink Port Speeds'},
        'softwareDescription': {
            'id': 1228,
            'longDescription': 'Redhat EL 5.10-64',
            'referenceCode': 'REDHAT_5_64'
        },
        'prices': [{'id': 1122,
                    'hourlyRecurringFee': 0.0,
                    'recurringFee': 0.0,
                    'categories': [{'id': 26,
                                    'name': 'Uplink Port Speeds',
                                    'categoryCode': 'port_speed'}]}],
    },
    {
        'id': 2233,
        'keyName': 'KeyName02',
        'capacity': '1000',
        'description': 'Public & Private Networks',
        'itemCategory': {'categoryCode': 'Uplink Port Speeds'},
        'prices': [{'id': 4477,
                    'hourlyRecurringFee': 0.0,
                    'recurringFee': 0.0,
                    'categories': [{'id': 26,
                                    'name': 'Uplink Port Speeds',
                                    'categoryCode': 'port_speed'}]}],
    },
    {
        'id': 1239,
        'keyName': 'KeyName03',
        'capacity': '2',
        'description': 'RAM',
        'itemCategory': {'categoryCode': 'RAM'},
        'prices': [{'id': 1133,
                    'hourlyRecurringFee': 0.0,
                    'recurringFee': 0.0,
                    'categories': [{'id': 3,
                                    'name': 'RAM',
                                    'categoryCode': 'ram'}]}],
    }
]

verifyOrderDH = {
    'preTaxSetup': '0',
    'storageGroups': [],
    'postTaxRecurring': '3.164',
    'billingOrderItemId': '',
    'presetId': '',
    'hardware': [
        {
            'domain': 't.com',
            'hostname': 't',
            'bareMetalInstanceFlag': '',
            'hardwareStatusId': '',
            'primaryBackendNetworkComponent': {
                'router': {
                    'id': 51218
                },
                'networkVlanId': ''
            },
            'accountId': ''
        }
    ],
    'prices': [
        {
            'itemId': 10195,
            'setupFee': '0',
            'recurringFee': '0',
            'hourlyRecurringFee': '3.164',
            'oneTimeFee': '0',
            'id': 200269,
            'item': {
                'thirdPartyPolicyAssignments': [],
                'capacity': '56',
                'description': '56 Cores X 242 RAM X 1.2 TB',
                'bundle': [
                    {
                        'category': {
                            'categoryCode': 'dedicated_host_ram',
                            'id': 850,
                            'name': 'Dedicated Host RAM'
                        },
                        'itemPriceId': 200301,
                        'itemPrice': {
                            'itemId': 10199,
                            'setupFee': '0',
                            'recurringFee': '0',
                            'hourlyRecurringFee': '0',
                            'oneTimeFee': '0',
                            'id': 200301,
                            'laborFee': '0'
                        },
                        'bundleItemId': 10195,
                        'bundleItem': {
                            'units': 'CORE',
                            'keyName': '56_CORES_X_242_RAM_X_1_4_TB',
                            'capacity': '56',
                            'description': '56 Cores X 242 RAM X 1.2 TB',
                            'id': 10195
                        },
                        'id': 41763
                    },
                    {
                        'category': {
                            'categoryCode': 'dedicated_host_disk',
                            'id': 851,
                            'name': 'Dedicated Host Disk'
                        },
                        'itemPriceId': 200299,
                        'itemPrice': {
                            'itemId': 10197,
                            'setupFee': '0',
                            'recurringFee': '0',
                            'hourlyRecurringFee': '0',
                            'oneTimeFee': '0',
                            'id': 200299,
                            'laborFee': '0'
                        },
                        'bundleItemId': 10195,
                        'bundleItem': {
                            'units': 'CORE',
                            'keyName': '56_CORES_X_242_RAM_X_1_4_TB',
                            'capacity': '56',
                            'description': '56 Cores X 242 RAM X 1.2 TB',
                            'id': 10195
                        },
                        'id': 41761
                    }
                ],
                'keyName': '56_CORES_X_242_RAM_X_1_4_TB',
                'units': 'CORE',
                'id': 10195
            },
            'laborFee': '0',
            'categories': [
                {
                    'categoryCode': 'dedicated_virtual_hosts',
                    'id': 848,
                    'name': 'Dedicated Host'
                }
            ]
        }
    ],
    'sendQuoteEmailFlag': '',
    'packageId': 813,
    'useHourlyPricing': True,
    'preTaxRecurringMonthly': '0',
    'message': '',
    'preTaxRecurring': '3.164',
    'primaryDiskPartitionId': '',
    'locationObject': {
        'id': 138124,
        'name': 'dal05',
        'longName': 'Dallas 5'
    },
    'taxCompletedFlag': False,
    'isManagedOrder': '',
    'imageTemplateId': '',
    'postTaxRecurringMonthly': '0',
    'resourceGroupTemplateId': '',
    'postTaxSetup': '0',
    'sshKeys': [],
    'location': '138124',
    'stepId': '',
    'proratedInitialCharge': '0',
    'totalRecurringTax': '0',
    'paymentType': '',
    'resourceGroupId': '',
    'sourceVirtualGuestId': '',
    'bigDataOrderFlag': False,
    'extendedHardwareTesting': '',
    'preTaxRecurringHourly': '3.164',
    'postTaxRecurringHourly': '3.164',
    'currencyShortName': 'USD',
    'containerSplHash': '000000003699c54000007f38ef8b0102',
    'proratedOrderTotal': '0',
    'serverCoreCount': '',
    'privateCloudOrderFlag': False,
    'totalSetupTax': '0',
    'quantity': 1
}

itemsLoadbal = [
    {
        "capacity": "0",
        "description": "Load Balancer as a Service",
        "id": 10043,
        "keyName": "LOAD_BALANCER_AS_A_SERVICE",
        "itemCategory": {
            "categoryCode": "load_balancer_as_a_service",
            "id": 1116,
            "name": "Load Balancer As A Service",
        },
        "prices": [
            {
                "hourlyRecurringFee": "0",
                "id": 199447,
                "locationGroupId": '',
                "recurringFee": "0",
            }
        ]
    },
    {
        "capacity": "0",
        "description": "Load Balancer Uptime",
        "id": 10785,
        "keyName": "LOAD_BALANCER_UPTIME",
        "itemCategory": {
            "categoryCode": "load_balancer_uptime",
            "id": 1119,
            "name": "Load Balancer Uptime",
        },
        "prices": [
            {
                "hourlyRecurringFee": ".028",
                "id": 205913,
                "locationGroupId": 507,
            }]}
]

regionsLoadbal = [{'description': 'WDC01 - Washington, DC - East Coast U.S.',
                   'keyname': 'WASHINGTON_DC',
                   'location': {'location': {'id': 37473,
                                             'longName': 'Washington 1',
                                             'name': 'wdc01',
                                             "groups": [
                                                 {
                                                     "description": "Location Group 4",
                                                     "id": 507,
                                                     "locationGroupTypeId": 82,
                                                     "name": "Location Group 4",
                                                     "locationGroupType": {
                                                         "name": "PRICING"
                                                     }
                                                 },
                                                 {
                                                     "description": "COS Cross Region - EU",
                                                     "id": 1303,
                                                     "locationGroupTypeId": 82,
                                                     "name": "eu",
                                                     "locationGroupType": {
                                                         "name": "PRICING"
                                                     }
                                                 },
                                                 {
                                                     "description": "COS Regional Frankfurt",
                                                     "id": 1783,
                                                     "locationGroupTypeId": 82,
                                                     "name": "eu-de",
                                                     "locationGroupType": {
                                                         "name": "PRICING"
                                                     }
                                                 }
                                             ]
                                             }},
                   'sortOrder': 10}]

getAllObjectsLoadbal = [
    {
        "id": 805,
        "keyName": "LBAAS",
        "name": "Load Balancer As A Service (LBaaS)",
        "items": itemsLoadbal,
        "regions": regionsLoadbal
    }
]

getAllObjectsDH = [{
    "subDescription": "Dedicated Host",
    "name": "Dedicated Host",
    "items": [{
        "capacity": "56",
        "description": "56 Cores X 242 RAM X 1.2 TB",
        "bundleItems": [
            {
                "capacity": "1200",
                "keyName": "1_4_TB_LOCAL_STORAGE_DEDICATED_HOST_CAPACITY",
                "categories": [{
                    "categoryCode": "dedicated_host_disk"
                }]
            },
            {
                "capacity": "242",
                "keyName": "242_GB_RAM",
                "categories": [{
                    "categoryCode": "dedicated_host_ram"
                }]
            }
        ],
        "prices": [
            {
                "itemId": 10195,
                "setupFee": "0",
                "recurringFee": "2099",
                "tierMinimumThreshold": "",
                "hourlyRecurringFee": "3.164",
                "oneTimeFee": "0",
                "currentPriceFlag": "",
                "id": 200269,
                "sort": 0,
                "onSaleFlag": "",
                "laborFee": "0",
                "locationGroupId": "",
                "quantity": ""
            },
            {
                "itemId": 10195,
                "setupFee": "0",
                "recurringFee": "2161.97",
                "tierMinimumThreshold": "",
                "hourlyRecurringFee": "3.258",
                "oneTimeFee": "0",
                "currentPriceFlag": "",
                "id": 200271,
                "sort": 0,
                "onSaleFlag": "",
                "laborFee": "0",
                "locationGroupId": 503,
                "quantity": ""
            }
        ],
        "keyName": "56_CORES_X_242_RAM_X_1_4_TB",
        "id": 10195,
        "itemCategory": {
            "categoryCode": "dedicated_virtual_hosts"
        }
    }],
    "keyName": "DEDICATED_HOST",
    "unitSize": "",
    "regions": [{
        "location": {
            "locationPackageDetails": [{
                "isAvailable": 1,
                "locationId": 138124,
                "packageId": 813
            }],
            "location": {
                "statusId": 2,
                "priceGroups": [{
                    "locationGroupTypeId": 82,
                    "description": "CDN - North America - Akamai",
                    "locationGroupType": {
                        "name": "PRICING"
                    },
                    "securityLevelId": "",
                    "id": 1463,
                    "name": "NORTH-AMERICA-AKAMAI"
                }],
                "id": 138124,
                "name": "dal05",
                "longName": "Dallas 5"
            }
        },
        "keyname": "DALLAS05",
        "description": "DAL05 - Dallas",
        "sortOrder": 12
    }],
    "firstOrderStepId": "",
    "id": 813,
    "isActive": 1,
    "description": "Dedicated Host"
}]

getAllObjectsDHGpu = [{
    "subDescription": "Dedicated Host",
    "name": "Dedicated Host",
    "items": [{
        "capacity": "56",
        "description": "56 Cores x 360 RAM x 1.2 TB x 2 GPU P100 [encryption enabled]",
        "bundleItems": [
            {
                "capacity": "1200",
                "keyName": "1.2 TB Local Storage (Dedicated Host Capacity)",
                "categories": [{
                    "categoryCode": "dedicated_host_disk"
                }]
            },
            {
                "capacity": "242",
                "keyName": "2_GPU_P100_DEDICATED",
                "hardwareGenericComponentModel": {
                    "capacity": "16",
                    "id": 849,
                    "hardwareComponentType": {
                        "id": 20,
                        "keyName": "GPU"
                    }
                },
                "categories": [{
                    "categoryCode": "dedicated_host_ram"
                }]
            }
        ],
        "prices": [
            {
                "itemId": 10195,
                "setupFee": "0",
                "recurringFee": "2099",
                "tierMinimumThreshold": "",
                "hourlyRecurringFee": "3.164",
                "oneTimeFee": "0",
                "currentPriceFlag": "",
                "id": 200269,
                "sort": 0,
                "onSaleFlag": "",
                "laborFee": "0",
                "locationGroupId": "",
                "quantity": ""
            },
            {
                "itemId": 10195,
                "setupFee": "0",
                "recurringFee": "2161.97",
                "tierMinimumThreshold": "",
                "hourlyRecurringFee": "3.258",
                "oneTimeFee": "0",
                "currentPriceFlag": "",
                "id": 200271,
                "sort": 0,
                "onSaleFlag": "",
                "laborFee": "0",
                "locationGroupId": 503,
                "quantity": ""
            }
        ],
        "keyName": "56_CORES_X_484_RAM_X_1_5_TB_X_2_GPU_P100",
        "id": 10195,
        "itemCategory": {
            "categoryCode": "dedicated_virtual_hosts"
        }
    }],
    "keyName": "DEDICATED_HOST",
    "unitSize": "",
    "regions": [{
        "location": {
            "locationPackageDetails": [{
                "isAvailable": 1,
                "locationId": 138124,
                "packageId": 813
            }],
            "location": {
                "statusId": 2,
                "priceGroups": [{
                    "locationGroupTypeId": 82,
                    "description": "CDN - North America - Akamai",
                    "locationGroupType": {
                        "name": "PRICING"
                    },
                    "securityLevelId": "",
                    "id": 1463,
                    "name": "NORTH-AMERICA-AKAMAI"
                }],
                "id": 138124,
                "name": "dal05",
                "longName": "Dallas 5"
            }
        },
        "keyname": "DALLAS05",
        "description": "DAL05 - Dallas",
        "sortOrder": 12
    }],
    "firstOrderStepId": "",
    "id": 813,
    "isActive": 1,
    "description": "Dedicated Host"
}]

getRegions = [{
    "description": "WDC07 - Washington, DC",
    "keyname": "WASHINGTON07",
    "location": {
        "locationId": 2017603,
        "location": {
            "id": 2017603,
            "longName": "Washington 7",
            "name": "wdc07",
            "priceGroups": [
                {
                    "description": "COS Regional - US East",
                    "id": 1305,
                    "locationGroupTypeId": 82,
                    "name": "us-east",
                    "locationGroupType": {
                        "name": "PRICING"
                    }
                }
            ]
        }
    },
    "locations": [{
        "location": {
            "euCompliantFlag": False,
            "id": 2017603,
            "longName": "Washington 7",
            "name": "wdc07",
            "statusId": 2},
        "locationPackageDetails": [{
            "isAvailable": 1,
            "locationId": 2017603,
            "packageId": 46
        }]
    }]
}]

getItemPrices = [
    {
        "hourlyRecurringFee": ".093",
        "id": 204015,
        "recurringFee": "62",
        "categories": [
            {
                "categoryCode": "guest_core"
            }
        ],
        "item": {
            "description": "4 x 2.0 GHz or higher Cores",
            "id": 859,
            "keyName": "GUEST_CORES_4",
        },
        "pricingLocationGroup": {
            "id": 503,
            "locations": [
                {
                    "id": 449610,
                    "longName": "Montreal 1",
                    "name": "mon01",
                    "statusId": 2,
                    "regions": [
                        {
                            "description": "MON01 - Montreal",
                            "keyname": "MONTREAL",
                            "sortOrder": 94
                        }
                    ]
                },
                {
                    "id": 449618,
                    "longName": "Montreal 2",
                    "name": "mon02",
                    "statusId": 2
                },
                {
                    "id": 448994,
                    "longName": "Toronto 1",
                    "name": "tor01",
                    "statusId": 2
                },
                {
                    "id": 350993,
                    "longName": "Toronto 2",
                    "name": "tor02",
                    "statusId": 2
                },
                {
                    "id": 221894,
                    "longName": "Amsterdam 2",
                    "name": "ams02",
                    "statusId": 2,
                    "regions": [
                        {
                            "description": "AMS02 POP - Amsterdam",
                            "keyname": "AMSTERDAM02",
                            "sortOrder": 12
                        }
                    ]
                },
                {
                    "id": 265592,
                    "longName": "Amsterdam 1",
                    "name": "ams01",
                    "statusId": 2
                },
                {
                    "id": 814994,
                    "longName": "Amsterdam 3",
                    "name": "ams03",
                    "statusId": 2
                }
            ]
        }
    },
    {
        "hourlyRecurringFee": ".006",
        "id": 204663,
        "recurringFee": "4.1",
        "item": {
            "description": "100 GB (LOCAL)",
            "id": 3899,
            "keyName": "GUEST_DISK_100_GB_LOCAL_3",
        },
        "pricingLocationGroup": {
            "id": 503,
            "locations": [
                {
                    "id": 449610,
                    "longName": "Montreal 1",
                    "name": "mon01",
                    "statusId": 2
                },
                {
                    "id": 449618,
                    "longName": "Montreal 2",
                    "name": "mon02",
                    "statusId": 2
                },
                {
                    "id": 448994,
                    "longName": "Toronto 1",
                    "name": "tor01",
                    "statusId": 2
                },
                {
                    "id": 350993,
                    "longName": "Toronto 2",
                    "name": "tor02",
                    "statusId": 2
                },
                {
                    "id": 221894,
                    "longName": "Amsterdam 2",
                    "name": "ams02",
                    "statusId": 2
                },
                {
                    "id": 265592,
                    "longName": "Amsterdam 1",
                    "name": "ams01",
                    "statusId": 2
                },
                {
                    "id": 814994,
                    "longName": "Amsterdam 3",
                    "name": "ams03",
                    "statusId": 2
                }
            ]
        }
    },
    {
        "hourlyRecurringFee": ".217",
        "id": 204255,
        "recurringFee": "144",
        "item": {
            "description": "16 GB ",
            "id": 1017,
            "keyName": "RAM_16_GB",
        },
        "pricingLocationGroup": {
            "id": 503,
            "locations": [
                {
                    "id": 449610,
                    "longName": "Montreal 1",
                    "name": "mon01",
                    "statusId": 2
                },
                {
                    "id": 449618,
                    "longName": "Montreal 2",
                    "name": "mon02",
                    "statusId": 2
                },
                {
                    "id": 448994,
                    "longName": "Toronto 1",
                    "name": "tor01",
                    "statusId": 2
                },
                {
                    "id": 350993,
                    "longName": "Toronto 2",
                    "name": "tor02",
                    "statusId": 2
                },
                {
                    "id": 221894,
                    "longName": "Amsterdam 2",
                    "name": "ams02",
                    "statusId": 2
                },
                {
                    "id": 265592,
                    "longName": "Amsterdam 1",
                    "name": "ams01",
                    "statusId": 2
                },
                {
                    "id": 814994,
                    "longName": "Amsterdam 3",
                    "name": "ams03",
                    "statusId": 2
                }
            ]
        }
    }
]
getActivePresets = [
    {
        "description": "M1.64x512x25",
        "id": 799,
        "isActive": "1",
        "keyName": "M1_64X512X25",
        "name": "M1.64x512x25",
        "packageId": 835,
        "locations": [],
        "prices": [
            {
                "hourlyRecurringFee": "0",
                "id": 258963,
                "itemId": 8195,
                "recurringFee": "0",
                "setupFee": "0"
            }]
    },
    {
        "description": "M1.56x448x100",
        "id": 797,
        "isActive": "1",
        "keyName": "M1_56X448X100",
        "name": "M1.56x448x100",
        "packageId": 835,
        "locations": [],
        "prices": [
            {
                "hourlyRecurringFee": "0",
                "id": 698563,
                "itemId": 8195,
                "recurringFee": "0",
                "setupFee": "0"
            }]
    },
    {
        "description": "M1.64x512x100",
        "id": 801,
        "isActive": "1",
        "keyName": "M1_64X512X100",
        "name": "M1.64x512x100",
        "packageId": 835,
        "locations": [],
        "prices": [
            {
                "hourlyRecurringFee": "0",
                "id": 963258,
                "itemId": 8195,
                "recurringFee": "0",
                "setupFee": "0"
            }]
    }
]

getAccountRestrictedActivePresets = []

RESERVED_CAPACITY = [{"id": 1059}]
getItems_RESERVED_CAPACITY = [
    {
        'id': 12273,
        'keyName': 'B1_1X2_1_YEAR_TERM',
        'description': 'B1 1x2 1 year term',
        'capacity': 12,
        'itemCategory': {
            'categoryCode': 'reserved_capacity',
            'id': 2060,
            'name': 'Reserved Capacity',
            'quantityLimit': 20,
            'sortOrder': ''
        },
        'prices': [
            {
                'currentPriceFlag': '',
                'hourlyRecurringFee': '.032',
                'id': 217561,
                'itemId': 12273,
                'laborFee': '0',
                'locationGroupId': '',
                'onSaleFlag': '',
                'oneTimeFee': '0',
                'quantity': '',
                'setupFee': '0',
                'sort': 0,
                'tierMinimumThreshold': '',
                'categories': [
                    {
                        'categoryCode': 'reserved_capacity',
                        'id': 2060,
                        'name': 'Reserved Capacity',
                        'quantityLimit': 20,
                        'sortOrder': ''
                    }
                ]
            }
        ]
    }
]

getItems_1_IPV6_ADDRESS = [
    {
        'id': 4097,
        'keyName': '1_IPV6_ADDRESS',
        'itemCategory': {
            'categoryCode': 'pri_ipv6_addresses',
            'id': 325,
            'name': 'Primary IPv6 Addresses',
            'quantityLimit': 0,
            'sortOrder': 34
        },
        'prices': [
            {
                'currentPriceFlag': '',
                'hourlyRecurringFee': '0',
                'id': 17129,
                'itemId': 4097,
                'laborFee': '0',
                'locationGroupId': '',
                'onSaleFlag': '',
                'oneTimeFee': '0',
                'quantity': '',
                'recurringFee': '0',
                'setupFee': '0',
                'sort': 0,
                'tierMinimumThreshold': '',
                'categories': [
                    {
                        'categoryCode': 'pri_ipv6_addresses',
                        'id': 325,
                        'name': 'Primary IPv6 Addresses',
                        'quantityLimit': 0,
                        'sortOrder': 34
                    }
                ]
            }
        ]
    }
]

getObject = {
    'id': 200,
    'regions': [{'description': 'WDC01 - Washington, DC - East Coast U.S.',
                 'keyname': 'WASHINGTON_DC',
                 'location': {'location': {'id': 37473,
                                           'longName': 'Washington 1',
                                           'name': 'wdc01'}},
                 'sortOrder': 10}],
    'accountRestrictedActivePresets': [],
    'activePresets': [
        {
            'description': 'AC2.8x60x25',
            'id': 861,
            'isActive': '1',
            'keyName': 'AC2_8X60X25',
            'name': 'AC2.8x60x25',
            'packageId': 835
        },
        {
            'description': 'AC2.8x60x100',
            'id': 863,
            'isActive': '1',
            'keyName': 'AC2_8X60X100',
            'name': 'AC2.8x60x100',
            'packageId': 835
        }],
    "items": [{
        "capacity": "56",
        "description": "56 Cores x 360 RAM x 1.2 TB x 2 GPU P100 [encryption enabled]",
        "bundleItems": [
            {
                "capacity": "1200",
                "keyName": "1.2 TB Local Storage (Dedicated Host Capacity)",
                "categories": [{
                    "categoryCode": "dedicated_host_disk"
                }]
            },
            {
                "capacity": "242",
                "keyName": "2_GPU_P100_DEDICATED",
                "hardwareGenericComponentModel": {
                    "capacity": "16",
                    "id": 849,
                    "hardwareComponentType": {
                        "id": 20,
                        "keyName": "GPU"
                    }
                },
                "categories": [{
                    "categoryCode": "dedicated_host_ram"
                }, {
                    "capacity": "2",
                    "description": "2 x 2.0 GHz or higher Cores",
                    "keyName": "GUEST_CORES_2",
                    "attributes": [
                        {
                            "id": 8261,
                            "attributeTypeKeyName": "ORDER_SAVES_USAGE_FEES"
                        }
                    ],
                    "itemCategory": {
                        "categoryCode": "guest_core",
                        "id": 80
                    }}]
            }
        ],
        "prices": [
            {
                "itemId": 10195,
                "setupFee": "0",
                "recurringFee": "2099",
                "tierMinimumThreshold": "",
                "hourlyRecurringFee": "3.164",
                "oneTimeFee": "0",
                "currentPriceFlag": "",
                "id": 200269,
                "sort": 0,
                "onSaleFlag": "",
                "laborFee": "0",
                "locationGroupId": "",
                "quantity": ""
            },
            {
                "itemId": 10195,
                "setupFee": "0",
                "recurringFee": "2161.97",
                "tierMinimumThreshold": "",
                "hourlyRecurringFee": "3.258",
                "oneTimeFee": "0",
                "currentPriceFlag": "",
                "id": 200271,
                "sort": 0,
                "onSaleFlag": "",
                "laborFee": "0",
                "locationGroupId": 503,
                "quantity": ""
            }
        ],
        "keyName": "56_CORES_X_484_RAM_X_1_5_TB_X_2_GPU_P100",
        "id": 10195,
        "itemCategory": {
            "categoryCode": "dedicated_virtual_hosts"
        }
    }]}

getItems_vmware = [{
    "capacity": "2",
    "description": "VMware vSAN Enterprise Tier III 65 - 124 TB 6.x",
    "id": 9567,
    "itemTaxCategoryId": 166,
    "keyName": "VMWARE_VSAN_ENTERPRISE_TIER_III_65_124_TB_6_X_2",
    "softwareDescriptionId": 1979,
    "units": "CPU",
    "itemCategory": {
        "categoryCode": "software_license",
        "id": 438,
        "name": "Software License",
        "quantityLimit": 1,
    },
    "prices": [
        {
            "id": 245164,
            "itemId": 9567,
            "laborFee": "0",
            "locationGroupId": None,
            "recurringFee": "0",
            "setupFee": "0",
            "sort": 0,
        }
    ]}]

getItemsVLAN = [{
    "description": "Private Network Vlan",
    "id": 1072,
    "itemTaxCategoryId": 166,
    "keyName": "PRIVATE_NETWORK_VLAN",
    "itemCategory": {
        "categoryCode": "network_vlan",
        "id": 113,
        "name": "Network Vlan"},
    "prices": [{
        "id": 203707,
        "itemId": 1072,
        "laborFee": "0",
        "locationGroupId": 505,
        "oneTimeFee": "0",
        "recurringFee": "0",
        "setupFee": "0",
        "sort": 10,
    },
        {
            "id": 203727,
            "itemId": 1072,
            "laborFee": "0",
            "locationGroupId": 545,
            "oneTimeFee": "0",
            "recurringFee": "0",
            "setupFee": "0",
            "sort": 10,
    }]
}, {
    "description": "Public Network Vlan",
    "id": 1071,
    "itemTaxCategoryId": 166,
    "keyName": "PUBLIC_NETWORK_VLAN",
    "units": "N/A",
    "itemCategory": {
        "categoryCode": "network_vlan",
        "id": 113,
        "name": "Network Vlan",
    },
    "prices": [{
        "id": 203637,
        "itemId": 1071,
        "laborFee": "0",
        "locationGroupId": 509,
        "oneTimeFee": "0",
        "recurringFee": "0",
        "setupFee": "0",
        "sort": 10,
    },
        {
            "id": 203667,
            "itemId": 1071,
            "laborFee": "0",
            "locationGroupId": 545,
            "oneTimeFee": "0",
            "recurringFee": "0",
            "setupFee": "0",
            "sort": 10,
    }]
}
]

getAllObjectsIPSEC = [{
    "firstOrderStepId": 1,
    "id": 0,
    "isActive": 1,
    "keyName": "ADDITIONAL_PRODUCTS",
    "name": "Additional Products"}]

getItems_IPSEC = [{
    "description": "IPSEC - Standard",
    "id": 1092,
    "keyName": "IPSEC_STANDARD",
    "categories": [
        {
            "categoryCode": "network_tunnel",
            "id": 117,
            "name": "Network Tunnel",
            "quantityLimit": 0,
        },
        {
            "categoryCode": "one_time_charge",
            "id": 33,
            "name": "One Time Charge",
            "quantityLimit": 0,
        }
    ],
    "itemCategory": {
        "categoryCode": "network_tunnel",
        "id": 117,
        "name": "Network Tunnel",
        "quantityLimit": 0,
    },
    "prices": [
        {
            "id": 51379,
            "itemId": 1092,
            "laborFee": "0",
            "locationGroupId": 503,
            "oneTimeFee": "0",
            "recurringFee": "102",
            "setupFee": "0",
            "sort": 0,
        }
    ]
}]

getVSDiskItemPrices = [
    {
        "id": 2255,
        "locationGroupId": None,
        "categories": [
            {
                "categoryCode": "guest_disk1",
                "id": 82,
                "name": "Second Disk",
                "quantityLimit": 0,
                "sortOrder": None
            },
            {
                "categoryCode": "guest_disk2",
                "id": 92,
                "name": "Third Disk",
                "quantityLimit": 0,
                "sortOrder": None
            },
            {
                "categoryCode": "guest_disk3",
                "id": 93,
                "name": "Fourth Disk",
                "quantityLimit": 0,
                "sortOrder": None
            },
            {
                "categoryCode": "guest_disk4",
                "id": 116,
                "name": "Fifth Disk",
                "quantityLimit": 0,
                "sortOrder": None
            }
        ],
        "item": {
            "capacity": "10",
            "description": "10 GB (SAN)",
            "keyName": "GUEST_DISK_10_GB_SAN"
        }
    },
    {
        "id": 1639,
        "locationGroupId": None,
        "categories": [
            {
                "categoryCode": "guest_disk0",
                "id": 81,
                "name": "First Disk",
                "quantityLimit": 0,
                "sortOrder": None
            }
        ],
        "item": {
            "capacity": "10",
            "description": "10 GB (SAN)",
            "keyName": "GUEST_DISK_10_GB_SAN"
        }
    }
]

getItemsOS = [{
    "description": "Debian GNU/Linux 6.x Squeeze/Stable - LAMP Install (64 bit)",
    "id": 3975,
    "itemTaxCategoryId": 166,
    "keyName": "OS_DEBIAN_6_X_SQUEEZE_LAMP_64_BIT",
    "softwareDescriptionId": 881,
    "itemCategory": {
        "categoryCode": "os",
        "id": 12,
        "name": "Operating System",
    },
    "prices": [
        {
            "hourlyRecurringFee": "0",
            "id": 14056,
            "itemId": 3975,
            "laborFee": "0",
            "recurringFee": "0",
            "setupFee": "0",
            "packageReferences": [
                {
                    "id": 74660,
                    "itemPriceId": 14056,
                    "packageId": 46
                }
            ]
        }
    ]
}]



"""Order and Cancel LBaaS instances."""
import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.exceptions import SoftLayerAPIError
from SoftLayer import utils


# pylint: disable=unused-argument
def parse_proto(ctx, param, value):
    """Parses the frontend and backend cli options"""
    proto = {'protocol': 'HTTP', 'port': 80}
    splitout = value.split(':')
    if len(splitout) != 2:
        raise exceptions.ArgumentError(f"{param}={value} is not properly formatted.")
    proto['protocol'] = splitout[0]
    proto['port'] = int(splitout[1])
    return proto


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--name', '-n', help='Label for this loadbalancer.', required=True)
@click.option('--datacenter', '-d', help='Datacenter shortname (dal13).', required=True)
@click.option('--label', '-l', help='A descriptive label for this loadbalancer.')
@click.option('--frontend', '-f', required=True, default='HTTP:80', show_default=True, callback=parse_proto,
              help='PROTOCOL:PORT string for incoming internet connections.')
@click.option('--backend', '-b', required=True, default='HTTP:80', show_default=True, callback=parse_proto,
              help='PROTOCOL:PORT string for connecting to backend servers.')
@click.option('--method', '-m', help="Balancing Method.", default='ROUNDROBIN', show_default=True,
              type=click.Choice(['ROUNDROBIN', 'LEASTCONNECTION', 'WEIGHTED_RR']))
@click.option('--subnet', '-s', required=True,
              help="Private subnet Id to order the LB on. See `slcli lb order-options`")
@click.option('--public', is_flag=True, default=False, show_default=True, help="Use a Public to Public loadbalancer.")
@click.option('--verify', is_flag=True, default=False, show_default=True,
              help="Only verify an order, dont actually create one.")
@environment.pass_env
def order(env, **args):
    """Creates a LB. Protocols supported are TCP, HTTP, and HTTPS."""

    mgr = SoftLayer.LoadBalancerManager(env.client)
    network = SoftLayer.NetworkManager(env.client)

    datacenter = network.get_datacenter(datacenter=args.get('datacenter'))
    if datacenter:
        location = {'id': datacenter[0]['id']}
    else:
        raise exceptions.CLIHalt(f'Datacenter {datacenter} was not found')

    name = args.get('name')
    description = args.get('label', None)

    backend = args.get('backend')
    frontend = args.get('frontend')
    protocols = [
        {
            "backendPort": backend.get('port'),
            "backendProtocol": backend.get('protocol'),
            "frontendPort": frontend.get('port'),
            "frontendProtocol": frontend.get('protocol'),
            "loadBalancingMethod": args.get('method'),
            "maxConn": 1000
        }
    ]

    # remove verify=True to place the order
    receipt = mgr.order_lbaas(location, name, description, protocols, args.get('subnet'),
                              public=args.get('public'), verify=args.get('verify'))
    table = parse_receipt(receipt)
    env.fout(table)


def parse_receipt(receipt):
    """Takes an order receipt and nicely formats it for cli output"""
    table = formatting.KeyValueTable(['Item', 'Cost'], title=f"Order: {receipt.get('orderId', 'Quote')}")
    if receipt.get('prices'):
        for price in receipt.get('prices'):
            table.add_row([price['item']['description'], price['hourlyRecurringFee']])
    elif receipt.get('orderDetails'):
        for price in receipt['orderDetails']['prices']:
            table.add_row([price['item']['description'], price['hourlyRecurringFee']])

    return table


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--datacenter', '-d', help="Show only selected datacenter, use shortname (dal13) format.")
@environment.pass_env
def order_options(env, datacenter):
    """Prints options for order a LBaaS"""
    print("Prints options for ordering")
    mgr = SoftLayer.LoadBalancerManager(env.client)
    net_mgr = SoftLayer.NetworkManager(env.client)
    package = mgr.lbaas_order_options()

    if not datacenter:
        data_table = formatting.KeyValueTable(['Datacenters', 'City'])
        for region in package['regions']:
            data_table.add_row([region['description'].split('-')[0], region['description'].split('-')[1]])
            # print(region)
        env.fout(data_table)
        click.secho("Use `slcli lb order-options --datacenter <DC>` "
                    "to find pricing information and private subnets for that specific site.")

    else:
        for region in package['regions']:
            dc_name = utils.lookup(region, 'location', 'location', 'name')

            # Skip locations if they are not the one requested.
            if datacenter and dc_name != datacenter.lower():
                continue

            l_groups = []
            for group in region['location']['location']['groups']:
                l_groups.append(group.get('id'))

            # Price lookups
            prices = []
            price_table = formatting.KeyValueTable(['KeyName', 'Cost'], title='Prices')
            for item in package['items']:
                i_price = {'keyName': item['keyName']}
                for price in item.get('prices', []):
                    if not price.get('locationGroupId'):
                        i_price['default_price'] = price.get('hourlyRecurringFee')
                    elif price.get('locationGroupId') in l_groups:
                        i_price['region_price'] = price.get('hourlyRecurringFee')
                prices.append(i_price)
            for price in prices:
                if price.get('region_price'):
                    price_table.add_row([price.get('keyName'), price.get('region_price')])
                else:
                    price_table.add_row([price.get('keyName'), price.get('default_price')])

            # Vlan/Subnet Lookups
            mask = "mask[networkVlan,podName,addressSpace]"
            subnets = net_mgr.list_subnets(datacenter=dc_name, network_space='PRIVATE', mask=mask)
            subnet_table = formatting.Table(['Id', 'Subnet', 'Vlan'], title='Private subnet')

            for subnet in subnets:
                # Only show these types, easier to filter here than in an API call.
                if subnet.get('subnetType') != 'PRIMARY' and \
                        subnet.get('subnetType') != 'ADDITIONAL_PRIMARY':
                    continue
                space = f"{subnet.get('networkIdentifier')}/{subnet.get('cidr')}"
                vlan = f"{subnet['podName']}.{subnet['networkVlan']['vlanNumber']}"
                subnet_table.add_row([subnet.get('id'), space, vlan])

            env.fout(price_table)
            env.fout(subnet_table)


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--force', default=False, is_flag=True, help="Force cancel LBaaS Instance without confirmation")
@environment.pass_env
def cancel(env, identifier, force):
    """Cancels a LBaaS instance"""

    mgr = SoftLayer.LoadBalancerManager(env.client)
    uuid, _ = mgr.get_lbaas_uuid_id(identifier)

    if not force:
        if not (env.skip_confirmations or
                formatting.confirm("This will cancel the LBaaS Instance and cannot be undone. Continue?")):
            raise exceptions.CLIAbort('Aborted')

    try:
        mgr.cancel_lbaas(uuid)
        click.secho(f"LB {identifier} canceled succesfully.", fg='green')
    except SoftLayerAPIError as exception:
        click.secho(f"ERROR: {exception.faultString}", fg='red')



"""List block storage volumes."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import columns as column_helper
from SoftLayer.CLI import environment
from SoftLayer.CLI import storage_utils

COLUMNS = [
    column_helper.Column('id', ('id',), mask="id"),
    column_helper.Column('username', ('username',), mask="username"),
    column_helper.Column('datacenter',
                         ('serviceResource', 'datacenter', 'name'),
                         mask="serviceResource.datacenter.name"),
    column_helper.Column(
        'storage_type',
        lambda b: b['storageType']['keyName'].split('_').pop(0)
        if 'storageType' in b and 'keyName' in b['storageType']
           and isinstance(b['storageType']['keyName'], str)
        else '-',
        mask="storageType.keyName"),
    column_helper.Column('capacity_gb', ('capacityGb',), mask="capacityGb"),
    column_helper.Column('IOPs', ('provisionedIops',), mask="provisionedIops"),
    column_helper.Column('ip_addr', ('serviceResourceBackendIpAddress',),
                         mask="serviceResourceBackendIpAddress"),
    column_helper.Column('lunId', ('lunId',), mask="lunId"),
    column_helper.Column('active_transactions', ('activeTransactionCount',),
                         mask="activeTransactionCount"),
    column_helper.Column('rep_partner_count', ('replicationPartnerCount',),
                         mask="replicationPartnerCount"),
    column_helper.Column(
        'created_by',
        ('billingItem', 'orderItem', 'order', 'userRecord', 'username')),
    column_helper.Column('notes', ('notes',), mask="notes"),
]

DEFAULT_COLUMNS = [
    'id',
    'username',
    'datacenter',
    'storage_type',
    'capacity_gb',
    'IOPs',
    'ip_addr',
    'lunId',
    'active_transactions',
    'rep_partner_count',
    'notes'
]


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--username', '-u', help='Volume username')
@click.option('--datacenter', '-d', help='Datacenter shortname')
@click.option('--order', '-o', type=int, help='Filter by ID of the order that purchased the block storage')
@click.option('--storage-type',
              help='Type of storage volume',
              type=click.Choice(['performance', 'endurance']))
@click.option('--sortby', help='Column to sort by', default='username')
@click.option('--columns',
              callback=column_helper.get_formatter(COLUMNS),
              help=f"Columns to display. Options: {', '.join(column.name for column in COLUMNS)}",
              default=','.join(DEFAULT_COLUMNS))
@environment.pass_env
def cli(env, sortby, columns, datacenter, username, storage_type, order):
    """List block storage.

    Example::
        slcli block volume-list -d dal09 -t endurance --sortby capacity_gb
        This command lists all endurance volumes on current account \
that are located at dal09, and sorts them by capacity.
"""

    block_manager = SoftLayer.BlockStorageManager(env.client)
    block_volumes = block_manager.list_block_volumes(datacenter=datacenter,
                                                     username=username,
                                                     storage_type=storage_type,
                                                     order=order,
                                                     mask=columns.mask())

    table = storage_utils.build_output_table(env, block_volumes, columns, sortby)
    env.fout(table)



"""Get user access details a virtual server."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """Get user access details a virtual server."""

    vsi = SoftLayer.VSManager(env.client)
    access_logs = vsi.browser_access_log(identifier)

    table = formatting.KeyValueTable(['Username', 'Source IP', 'Port', 'Date', 'Event', 'Message'])
    for log in access_logs:
        table.add_row([log.get('username'), log.get('sourceIp'), log.get('sourcePort'), log.get('createDate'),
                       log.get('eventType'), log.get('message')])

    env.fout(table)



"""Get details about a VLAN."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer import utils


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--no-vs', is_flag=True,
              help="Hide virtual server listing")
@click.option('--no-hardware', is_flag=True,
              help="Hide hardware listing")
@click.option('--no-trunks', is_flag=True,
              help="Hide devices with trunks")
@environment.pass_env
def cli(env, identifier, no_vs, no_hardware, no_trunks):
    """Get details about a VLAN."""
    mgr = SoftLayer.NetworkManager(env.client)

    vlan_id = helpers.resolve_id(mgr.resolve_vlan_ids, identifier, 'VLAN')

    mask = """mask[
firewallInterfaces, datacenter[name, longName], primaryRouter[fullyQualifiedDomainName],
totalPrimaryIpAddressCount,
networkSpace, id, vlanNumber, fullyQualifiedName, name,
hardware[id, hostname, domain, primaryIpAddress, primaryBackendIpAddress, tagReferences],
subnets[id, networkIdentifier, netmask, gateway, subnetType, usableIpAddressCount],
virtualGuests[id, hostname, domain, primaryIpAddress, primaryBackendIpAddress],
networkVlanFirewall[id,fullyQualifiedDomainName,primaryIpAddress],
attachedNetworkGateway[id,name,networkFirewall],
networkComponentTrunks[
    networkComponent[
        downlinkComponent[
            networkComponentGroup[membersDescription],
            hardware[tagReferences]
        ]
    ]
]
]"""

    vlan = mgr.get_vlan(vlan_id, mask=mask)

    table = formatting.KeyValueTable(['name', 'value'])
    table.align['name'] = 'r'
    table.align['value'] = 'l'

    table.add_row(['id', vlan.get('id')])
    table.add_row(['number', vlan.get('vlanNumber')])
    table.add_row(['datacenter', utils.lookup(vlan, 'datacenter', 'longName')])
    table.add_row(['primary_router', utils.lookup(vlan, 'primaryRouter', 'fullyQualifiedDomainName')])
    table.add_row(['Gateway/Firewall', get_gateway_firewall(vlan)])

    if vlan.get('subnets'):
        subnet_table = formatting.Table(['id', 'identifier', 'netmask', 'gateway', 'type', 'usable ips'])
        for subnet in vlan.get('subnets'):
            subnet_table.add_row([subnet.get('id'),
                                  subnet.get('networkIdentifier'),
                                  subnet.get('netmask'),
                                  subnet.get('gateway') or formatting.blank(),
                                  subnet.get('subnetType'),
                                  subnet.get('usableIpAddressCount')])
        # subnets.append(subnet_table)
        table.add_row(['subnets', subnet_table])
    else:
        table.add_row(['subnets', '-'])

    server_columns = ['hostname', 'domain', 'public_ip', 'private_ip']

    if not no_vs:
        if vlan.get('virtualGuests'):
            vs_table = formatting.KeyValueTable(server_columns)
            for vsi in vlan['virtualGuests']:
                vs_table.add_row([vsi.get('hostname'),
                                  vsi.get('domain'),
                                  vsi.get('primaryIpAddress'),
                                  vsi.get('primaryBackendIpAddress')])
            table.add_row(['vs', vs_table])
        else:
            table.add_row(['vs', '-'])

    if not no_hardware:
        if vlan.get('hardware'):
            hw_table = formatting.Table(server_columns)
            for hardware in vlan['hardware']:
                hw_table.add_row([hardware.get('hostname'),
                                  hardware.get('domain'),
                                  hardware.get('primaryIpAddress'),
                                  hardware.get('primaryBackendIpAddress')])
            table.add_row(['hardware', hw_table])
        else:
            table.add_row(['hardware', '-'])

    if not no_trunks:
        if vlan.get('networkComponentTrunks'):
            trunks = filter_trunks(vlan.get('networkComponentTrunks'))
            trunks_table = formatting.Table(['device', 'port', 'tags'])
            for trunk in trunks:
                trunks_table.add_row(get_trunk_row(trunk))
            table.add_row(['trunks', trunks_table])
        else:
            table.add_row(['trunks', '-'])

    env.fout(table)


def get_trunk_row(trunk: dict) -> list:
    """Parses a vlan trunk and returns a table row for it"""
    dl_component = utils.lookup(trunk, 'networkComponent', 'downlinkComponent')
    row = [
        utils.lookup(dl_component, 'hardware', 'fullyQualifiedDomainName'),
        utils.lookup(dl_component, 'networkComponentGroup', 'membersDescription'),
        formatting.tags(utils.lookup(dl_component, 'hardware', 'tagReferences'))
    ]
    return row


def get_gateway_firewall(vlan):
    """Gets the name of a gateway/firewall from a VLAN. """

    firewall = utils.lookup(vlan, 'networkVlanFirewall', 'fullyQualifiedDomainName')
    if firewall:
        return firewall
    gateway = utils.lookup(vlan, 'attachedNetworkGateway', 'name')
    if gateway:
        return gateway
    return formatting.blank()


def filter_trunks(trunks):
    """Filter duplicates devices with trunks of the vlan."""
    trunk_filters = []
    hardware_id = []
    for trunk in trunks:
        if utils.lookup(trunk, 'networkComponent', 'downlinkComponent', 'hardwareId') not in hardware_id:
            trunk_filters.append(trunk)
            hardware_id.append(utils.lookup(trunk, 'networkComponent', 'downlinkComponent', 'hardwareId'))
    return trunk_filters



"""Edit details of an image."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--add/--remove', default=True,
              help="To add or remove Datacenter")
@click.argument('locations', nargs=-1, required=True)
@environment.pass_env
def cli(env, identifier, add, locations):
    """Add/Remove datacenter of an image."""

    image_mgr = SoftLayer.ImageManager(env.client)
    image_id = helpers.resolve_id(image_mgr.resolve_ids, identifier, 'image')

    if add:
        result = image_mgr.add_locations(image_id, locations)
    else:
        result = image_mgr.remove_locations(image_id, locations)

    env.fout(result)



"""List options for creating Reserved Capacity"""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers.vs_capacity import CapacityManager as CapacityManager


@click.command(cls=SLCommand)
@environment.pass_env
def cli(env):
    """List options for creating Reserved Capacity"""
    manager = CapacityManager(env.client)
    items = manager.get_create_options()

    items.sort(key=lambda term: int(term['capacity']))
    table = formatting.Table(["KeyName", "Description", "Term", "Default Hourly Price Per Instance"],
                             title="Reserved Capacity Options")
    table.align["Hourly Price"] = "l"
    table.align["Description"] = "l"
    table.align["KeyName"] = "l"
    for item in items:
        table.add_row([
            item['keyName'], item['description'], item['capacity'], get_price(item)
        ])
    env.fout(table)

    regions = manager.get_available_routers()
    location_table = formatting.Table(['Location', 'POD', 'BackendRouterId'], 'Orderable Locations')
    for region in regions:
        for location in region['locations']:
            for pod in location['location']['pods']:
                location_table.add_row([region['keyname'], pod['backendRouterName'], pod['backendRouterId']])
    env.fout(location_table)


def get_price(item):
    """Finds the price with the default locationGroupId"""
    the_price = "No Default Pricing"
    for price in item.get('prices', []):
        if not price.get('locationGroupId'):
            the_price = f"{float(price['hourlyRecurringFee']):.4f}"
    return the_price
